#declare Font_Round_Ver=version
#version 2.0

// FNTRND.INC v2.0
// 
// Round fonts for use with POV-RAY v2.0
//
// Written by:  Howard G. Shank
//              Aurum Software
//              PO Box 156
//              Ankeny,  IA  50021
//              515/964-0317
//
//              CSERVE: 72724, 3066
//
// Copyright is retained by Howard G. Shank
// with rights granted for use and distribution
// both private and commercial provided this copyright
// notice is distributed with this file and the author's
// credit remains intact.
//
//
// Description:
//
// Fully scalable and translatable fonts.
// Font is positioned with the lower left corner being <-1, -1, -1> 
// and the upper right corner being <8, 15, 1>
// All letters have a radius of 1.  By scaling the object, these points
// are modified accordingly.
//
// USAGE:
//      Object {
//              Font_Round_A
//              pigment { color Red }                   // or whatever
//              translate <0, 0, 0>                     // 0,0,0 does not move from original position
//                                                      // and whatever other options you intend to use.
//      } // end of Object block
//


// Begin upper case round font letters

//      Capital A
#declare Font_Round_A = union {
    cylinder { <0,0,0>, <0,10.5,0>, 1      // left side
    }

    cylinder { <7,0,0>, <7,10.5,0>, 1      // left side
    }

    torus { 3.5, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3.5, 10.5, 0>
        clipped_by {
            box { <-1, 10.5, -1>, <8, 15, 1> } 
        }
    }

    sphere { <0, 0, 0>, 1 }             // left foot
    sphere { <7, 0, 0>, 1 }             // right foot
    cylinder { <0,7.5,0>, <7,7.5,0>, 1 }            // cross beam
  
}   // end union `A

//      Capital B
#declare Font_Round_B = union {

    sphere {                        // B/L corner
        <0, 0, 0>, 1
    }

    sphere {                        // T/L corner
        <0, 14, 0>, 1
    }
    
    torus { 3.5, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3.5, 10.5, 0>
        clipped_by {
            box { <3.5, 7, -1>, <10, 15, 1> }
        }
    }   // end torus

    torus { 3.5, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3.5, 3.5, 0>
        clipped_by {
            box { <3.5, -1, -1>, <10, 8, 1> }
        }
    }   // end torus

    cylinder { <0,0,0>, <0,14,0>, 1  // B/L to T/L side
    }

    cylinder { <0,14,0>, <4,14,0>, 1  // Top beam
    }

    cylinder { <0,7,0>, <4,7,0>, 1  // Middle beam
    }

    cylinder { <0,0,0>, <4,0,0>, 1  // Bottom beam
    }
}   // end union `B


//      Capital C
#declare Font_Round_C = union {
    sphere { <7,3.5,0> 1
    }   // end sphere
    
    sphere { <7,10.5,0> 1
    }   // end sphere

    cylinder { <0,3.5,0>, <0,10.5,0>, 1
    }

    torus { 3.5, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3.5, 10.5, 0>
        clipped_by {
            box { <-1, 10.5, -1>, <8, 15, 1> } 
        }
    }

    torus { 3.5, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3.5, 3.5, 0>
        clipped_by {
            box { <-1, -1, -1>, <8, 4, 1> }
        }
    }   // end torus

}   // end union `C


//      Capital D
#declare Font_Round_D = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/R
    }   // end sphere

    cylinder { <0,0,0>, <0,14,0>, 1     // left side
    }

    cylinder { <7,3.5,0>, <7,10.5,0>, 1      // right side
    }

    cylinder { <0, 14, 0>, <3.5, 14, 0>, 1      // top side
    }

    cylinder { <0,0,0>, <3.5, 0, 0>, 1      // bottom side
    }

    torus { 3.5, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3.5, 10.5, 0>
        clipped_by {
            box { <3.5, 10.5, -1>, <8, 15, 1> } 
        }
    }

    torus { 3.5, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3.5, 3.5, 0>
        clipped_by {
            box { <3.5, -1, -1>, <8, 3.5, 1> }
        }
    }   // end torus

}   // end union `D



#declare Font_Round_E = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    sphere { <7,7,0> 1         // M/R
    }   // end sphere

    sphere { <7,0,0> 1         // B/R
    }   // end sphere

    cylinder { <0,0,0>, <0,14,0>, 1     // left side
    }

    cylinder { <0, 14, 0>, <7, 14, 0>, 1      // top beam
    }

    cylinder { <0, 7, 0>, <7, 7, 0>, 1      // middle beam
    }
    cylinder { <0,0,0>, <7, 0, 0>, 1      // bottom beam
    }

}   // end union `E


#declare Font_Round_F = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    sphere { <5,7,0> 1         // M/R
    }   // end sphere


    cylinder { <0,0,0>, <0,14,0>, 1     // left side
    }

    cylinder { <0, 14, 0>, <7, 14, 0>, 1      // top beam
    }

    cylinder { <0, 7, 0>, <5, 7, 0>, 1      // middle beam
    }

}   // end union `F



#declare Font_Round_G = union {
    sphere { <7,3.5,0> 1              // b/r
    }   // end sphere

    sphere { <7,7,0> 1              // b/r
    }   // end sphere
    
    sphere { <7,10.5,0> 1              // t/r
    }   // end sphere

    sphere { <4,7,0> 1              // left end of beam
    }   // end sphere

    cylinder { <0,3.5,0>, <0,10.5,0>, 1      // left side
    }

    cylinder { <7,3.5,0>, <7,7,0>, 1      //right side
    }

    cylinder { <4,7,0>, <7,7,0>, 1      //bottom beam
    }

    torus { 3.5, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3.5, 10.5, 0>
        clipped_by {
            box { <-1, 10.5, -1>, <8, 15, 1> } 
        }
    }

    torus { 3.5, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3.5, 3.5, 0>
        clipped_by {
            box { <-1, -1, -1>, <8, 3.5, 1> }
        }
    }   // end torus

}   // end union `G



#declare Font_Round_H = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    cylinder { <0,0,0>, <0,14,0>, 1     // left side
    }

    cylinder { <7,0,0>, <7,14,0>, 1      // right side
    }

    cylinder { <0, 7, 0>, <7, 7, 0>, 1      // middle beam
    }


}   // end union `H



#declare Font_Round_I = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    cylinder { <0,0,0>, <0,14,0>, 1     // I beam (center)
    }

}   // end union `I



#declare Font_Round_J = union {
    sphere { <0,3.5,0> 1
    }   // end sphere
    
    sphere { <7,14,0> 1
    }   // end sphere

    cylinder { <7,3.5,0>, <7,14,0>, 1
    }


    torus { 3.5, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3.5, 3.5, 0>
        clipped_by {
            box { <-1, -1, -1>, <8, 3.5, 1> }
        }
    }   // end torus

}   // end union `J


#declare Font_Round_K = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    cylinder { <0, 0, 0>, <0, 14, 0>, 1     // B/L to U/L stroke
    }

    cylinder { <0, 7, 0>, <7, 14, 0>, 1     // c/l to u/r stroke
    }

    cylinder { <0, 7, 0>, <7, 0, 0>, 1      //  c/l to b/r stroke
    }

}   // end union `K




#declare Font_Round_L = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,0,0> 1         // B/R
    }   // end sphere

    cylinder { <0,0,0>, <0,14,0>, 1     // left side
    }

    cylinder { <0,0,0>, <7, 0, 0>, 1      // bottom beam
    }

}   // end union `L


#declare Font_Round_M = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    sphere { <3.5, 7, 0> 1         // C/C
    }   // end sphere

    cylinder { <0,14,0>, <3.5, 7, 0>, 1     // ul to c/c stroke
    }

    cylinder { <7,14,0>, <3.5, 7, 0>, 1     // ul to c/c stroke
    }

    cylinder { <7,0,0>, <7,14,0>, 1      // right side
    }

    cylinder { <0,0,0>, <0,14,0>, 1      // left side
    }


}   // end union `M


#declare Font_Round_N = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    cylinder { <0,14,0>, <7, 0, 0>, 1     // T/L to B/R stroke
    }

    cylinder { <7,0,0>, <7,14,0>, 1      // right side
    }

    cylinder { <0,0,0>, <0,14,0>, 1      // left side
    }


}   // end union `N



#declare Font_Round_O = union {
    cylinder { <0,3.5,0>, <0,10.5,0>, 1      // left side
    }

    cylinder { <7,3.5,0>, <7,10.5,0>, 1      // left side
    }

    torus { 3.5, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3.5, 10.5, 0>
        clipped_by {
            box { <-1, 10.5, -1>, <8, 15, 1> } 
        }
    }

    torus { 3.5, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3.5, 3.5, 0>
        clipped_by {
            box { <-1, -1, -1>, <8, 3.5, 1> }
        }
    }   // end torus

}   // end union `O


#declare Font_Round_P = union {
    sphere {                        // B/L corner
        <0, 0, 0>, 1
    }

    sphere {                        // T/L corner
        <0, 14, 0>, 1
    }
    
    torus { 3.5, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3.5, 10.5, 0>
        clipped_by {
            box { <3.5, 6, -1>, <8, 15, 1> }
        }
    }   // end torus


    cylinder { <0,0,0>, <0,14,0>, 1  // B/L to T/L side
    }

    cylinder { <0,14,0>, <3.5,14,0>, 1  // Top beam
    }

    cylinder { <0,7,0>, <3.5,7,0>, 1  // Middle beam
    }

}   // end union `P



#declare Font_Round_Q = union {

    cylinder { <0,3.5,0>, <0,10.5,0>, 1      // left side
    }
    cylinder { <7,3.5,0>, <7,10.5,0>, 1      // left side
    }

    sphere { <7,0,0>, 1                 // bottom of curly Q
    }
    sphere { <5,3,0>, 1                 // top of curly Q
    }
    cylinder { <7,0,0>, <5,3,0>, 1      // center of curly Q
    }

    torus { 3.5, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3.5, 10.5, 0>
        clipped_by {
            box { <-1, 10.5, -1>, <8, 15, 1> } 
        }
    }

    torus { 3.5, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3.5, 3.5, 0>
        clipped_by {
            box { <-1, -1, -1>, <8, 3.5, 1> }
        }
    }   // end torus

}   // end union `Q



#declare Font_Round_R = union {

    sphere {                        // B/L corner
        <0, 0, 0>, 1
    }

    sphere {                        // T/L corner
        <0, 14, 0>, 1
    }
    sphere {                        // B/R corner
        <7, 0, 0>, 1
    }

    cylinder { <7,0,0>, <3.5,7,0>, 1  // c/c to b/r arm
    }
    
    torus { 3.5, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3.5, 10.5, 0>
        clipped_by {
            box { <3.5, 6, -1>, <8, 15, 1> }
        }
    }   // end torus


    cylinder { <0,0,0>, <0,14,0>, 1  // B/L to T/L side
    }

    cylinder { <0,14,0>, <3.5,14,0>, 1  // Top beam
    }

    cylinder { <0,7,0>, <3.5,7,0>, 1  // Middle beam
    }

}   // end union `R


#declare Font_Round_S = union {
    sphere { <0,3.5,0> 1           // bottom left end cap
    }   // end sphere
    
    sphere { <7,10.5,0> 1           // top right end cap
    }   // end sphere
    
    union {
        torus { 3.5, 1 sturm              // top arch   top right
            rotate < 90, 0, 0>
            translate < 3.5, 10.5, 0>
            clipped_by {
                box { <-1, 10.5, -1>, <8, 15, 1> } 
            }
        }
        torus { 3.5, 1 sturm              // top arch top/side left
            rotate < 90, 0, 0>
            translate < 3.5, 10.5, 0>
            clipped_by {
                box { <-1, 6, -1>, <3.5, 10.5, 1> } 
            }
        }
    }   // end torus union 1

    union {
        torus { 3.5, 1 sturm              // bottom arch    bot/left
            rotate < 90, 0, 0>
            translate < 3.5, 3.5, 0>
            clipped_by {
                box { <-1, -1, -1>, <8, 3.5, 1> }
            }
        }   // end torus

        torus { 3.5, 1 sturm              // bottom arch    bot/side right
            rotate < 90, 0, 0>
            translate < 3.5, 3.5, 0>
            clipped_by {
                box { <3.5, 3.5, -1>, <8, 8, 1> }
            }
        }   // end torus
    } // end union torus 2

}   // end union `S


#declare Font_Round_T = union {
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    sphere { <3.5, 0, 0> 1         // B/C
    }   // end sphere

    cylinder { <3.5,0,0>, <3.5,14,0>, 1     // I beam (center)
    }

    cylinder { <0, 14, 0>, <7, 14, 0>, 1      // top beam
    }


}   // end union `T



#declare Font_Round_U = union {
    sphere { <0,14,0> 1         // T/L
    }   // end sphere
    
    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    cylinder { <7,3.5,0>, <7,14,0>, 1     // right side
    }

    cylinder { <0,3.5,0>, <0,14,0>, 1     // left side
    }

    torus { 3.5, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3.5, 3.5, 0>
        clipped_by {
            box { <-1, -1, -1>, <8, 3.5, 1> }
        }
    }   // end torus

}   // end union `U


#declare Font_Round_V = union {
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    sphere { <3.5, 0, 0> 1         // B/C
    }   // end sphere

    cylinder { <3.5,0,0>, <0,14,0>, 1     // left side
    }

    cylinder { <3.5,0,0>, <7,14,0>, 1     // right side
    }


}   // end union `V



#declare Font_Round_W = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    sphere { <3.5, 6, 0> 1         // C/C
    }   // end sphere

    cylinder { <0,0,0>, <3.5, 6, 0>, 1     // Bl to c/c stroke
    }

    cylinder { <7,0,0>, <3.5, 6, 0>, 1     // BR to c/c stroke
    }

    cylinder { <7,0,0>, <7,14,0>, 1      // right side
    }

    cylinder { <0,0,0>, <0,14,0>, 1      // left side
    }


}   // end union `W



#declare Font_Round_X = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    cylinder { <0,0,0>, <7,14,0>, 1     // bl to ur
    }

    cylinder { <7,0,0>, <0,14,0>, 1      // br to ul
    }
}   // end union `X


#declare Font_Round_Y = union {
    sphere { <0,14,0> 1          // B/L
    }   // end sphere
    
    sphere { <7,14,0> 1         // T/L
    }   // end sphere

    sphere { <3.5,0,0> 1          // B/C
    }   // end sphere
    
    cylinder { <0,14,0>, <3.5, 7, 0>, 1     // ul to c/c stroke
    }

    cylinder { <7,14,0>, <3.5, 7, 0>, 1     // ul to c/c stroke
    }

    cylinder { <3.5,0,0>, <3.5,7,0>, 1      // center stroke
    }
}   // end union `Y



#declare Font_Round_Z = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,14,0> 1         // T/L
    }   // end sphere

    sphere { <7,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <7,14,0> 1         // T/R
    }   // end sphere

    cylinder { <7,14,0>, <0, 0, 0>, 1     // T/L to B/R stroke
    }

    cylinder { <0,14,0>, <7,14,0>, 1      // top side
    }

    cylinder { <0,0,0>, <7,0,0>, 1      // bottom side
    }

}   // end union `Z

// end Font_Round Capital letters

// Begin lower case round font letters

//      Letter 'a
#declare Font_Round_a = union {
    cylinder { <6,0,0>, <6,7,0>, 1      // right side
    }

    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <-1, 4, -1>, <6, 8, 1> } 
        }
    }

    cylinder { <0, 3, 0>, <0, 4, 0>, 1 }

    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <-1, -1, -1>, <7, 3, 1> } 
        }
    }

    sphere { <6, 0, 0>, 1 }             // left foot
    sphere { <6, 7, 0>, 1 }             // right foot
  
}   // end union `a



//      'b
#declare Font_Round_b = union {

    sphere {                        // B/L corner
        <0, 0, 0>, 1
    }

    sphere {                        // T/L corner
        <0, 13, 0>, 1
    }
    
    torus { 3, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <1, -1, -1>, <7, 3, 1> }
        }
    }   // end torus
    cylinder { <6, 3, 0>, <6, 4, 0>, 1 }

    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <1, 4, -1>, <7, 8, 1> } 
        }
    }

    cylinder { <0,0,0>, <0,13,0>, 1  // B/L to T/L side
    }

}   // end union `b


//      'c
#declare Font_Round_c = union {
    sphere { <5.75,1.75,0> 1
    }   // end sphere
    
    sphere { <5.75,4.75,0>, 1
    }   // end sphere

    cylinder { <0,4,0>, <0,3,0>, 1 }

    torus { 3, 1 sturm              // top left side arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <-1, 4, -1>, <3, 8, 1> } 
        }
    }

    torus { 3, 1 sturm              // top right arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <3, 5, -1>, <7, 8, 1> } 
        }
    }

    torus { 3, 1 sturm              // bottom left side arch
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <-1, -1, -1>, <3, 3, 1> } 
        }
    }

    torus { 3, 1 sturm              // bot right arch
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <3, -1, -1>, <7, 2, 1> } 
        }
    }



}   // end union `c


//      'd
#declare Font_Round_d = union {
    sphere {                        // B/L corner
        <6, 0, 0>, 1
    }

    sphere {                        // T/L corner
        <6, 13, 0>, 1
    }
    
    torus { 3, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <-1, -1, -1>, <6, 3, 1> }
        }
    }   // end torus
    cylinder { <0, 3, 0>, <0, 4, 0>, 1 }

    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <-1, 4, -1>, <7, 8, 1> } 
        }
    }

    cylinder { <6,0,0>, <6,13,0>, 1  // B/L to T/L side
    }
}   // end union `d



#declare Font_Round_e = union {
    sphere { <6, 1.75, 0> 1
    }   // end sphere
    
    sphere { <6,4.25,0>, 1
    }   // end sphere

    cylinder { <6,4,0>, <0, 4, 0>, 1}   // cross beam

    cylinder { <0,4,0>, <0,3,0>, 1 }

    torus { 3, 1 sturm              // top left side arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <-1, 4, -1>, <3, 8, 1> } 
        }
    }

    torus { 3, 1 sturm              // top right arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <3, 5, -1>, <7, 8, 1> } 
        }
    }

    torus { 3, 1 sturm              // bottom left side arch
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <-1, -1, -1>, <3, 3, 1> } 
        }
    }

    torus { 3, 1 sturm              // bot right arch
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <3, -1, -1>, <7, 1.75, 1> } 
        }
    }



}   // end union `e


#declare Font_Round_f = union {

    sphere {                        // B/L corner
        <0, 0, 0>, 1
    }

    sphere {                        // T/L corner
        <6, 10, 0>, 1
    }
    
    torus { 3, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3, 10, 0>
        clipped_by {
            box { <-1, 10, -1>, <7, 14, 1> }
        }
    }   // end torus

    cylinder { <0,0,0>, <0,10,0>, 1  // B/L to T/L side
    }

    cylinder { <0,7,0>, <2.5,7,0>, 1}  // cross beam
    sphere { <2.5,7,0>,1} // end cross beam

}   // end union `f



#declare Font_Round_g = union {
    sphere { <6,13,0> 1              // t/l
    }   // end sphere

    sphere { <0,3,0> 1              // b/r
    }   // end sphere
    
    cylinder { <6,13,0>, <6,3,0>, 1      //bottom beam
    }

    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 9, 0>
        clipped_by {
            box { <-1, 5, -1>, <7, 9, 1> } 
        }
    }
    
    cylinder { <0, 9, 0>, <0, 10, 0>, 1 }
    
    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 10, 0>
        clipped_by {
            box { <-1, 10, -1>, <7, 14, 1> } 
        }
    }
    
    torus { 3, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <-1, -1, -1>, <7, 3, 1> }
        }
    }   // end torus

}   // end union `g



#declare Font_Round_h = union {

    sphere {                        // B/L corner
        <0, 0, 0>, 1
    }

    sphere {                        // T/L corner
        <0, 13, 0>, 1
    }
    
    torus { 3, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <0, 4, -1>, <7, 8, 1> }
        }
    }   // end torus

    cylinder { <0,0,0>, <0,13,0>, 1  // B/L to T/L side
    }

    cylinder { <6,4,0>, <6,0,0>, 1}
    sphere { <6,0,0>, 1}


}   // end union `h



#declare Font_Round_i = union {
    sphere { <0,0,0> 1          // Bot
    }   // end sphere
    
    sphere { <0,8,0> 1         // Top
    }   // end sphere
    
    sphere { <0,13,0> 1         // dot
    }   // end sphere

    cylinder { <0,0,0>, <0,8,0>, 1     // I beam (center)
    }

}   // end union `i



#declare Font_Round_j = union {


    torus { 2, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 4, 2, 0>
        clipped_by {
            box { <1, -1, -1>, <7, 2, 1> }
        }
    }   // end torus

    sphere { <2,2,0>, 1 }   // end hook
    cylinder { <6,2,0>, <6,8,0>, 1} // side
    sphere { <6,8,0>, 1}    // top stroke
    sphere { <6,13,0>, 1}   // dot

}   // end union `j


#declare Font_Round_k = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,13,0> 1         // T/L
    }   // end sphere

    sphere { <6,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <6,7,0> 1         // T/R
    }   // end sphere

    cylinder { <0, 0, 0>, <0, 13, 0>, 1     // B/L to U/L stroke
    }

    cylinder { <0, 4, 0>, <6, 7, 0>, 1     // c/l to u/r stroke
    }

    cylinder { <2, 5, 0>, <6, 0, 0>, 1      //  c/l to b/r stroke
    }

}   // end union `k




#declare Font_Round_l = union {
    sphere {                        // B/L corner
        <0, 0, 0>, 1
    }

    sphere {                        // T/L corner
        <0, 13, 0>, 1
    }
    
    cylinder { <0,0,0>, <0,13,0>, 1  // B/L to T/L side
    }

}   // end union `l


#declare Font_Round_m = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <4,0,0> 1         // B/C
    }   // end sphere

    sphere { <8,0,0> 1          // B/R
    }   // end sphere

    sphere { <0, 7, 0>, 1 }     // t/l

    cylinder { <0,0,0>, <0, 7, 0>, 1     // T/L to B/R stroke
    }
    cylinder { <4,0,0>, <4, 6, 0>, 1     // T/L to B/R stroke
    }
    cylinder { <8,0,0>, <8, 6, 0>, 1     // T/L to B/R stroke
    }
    union {
        torus { 2, 1 sturm              // bottom arch
            rotate < 90, 0, 0>
            translate < 2, 5, 0>
        }
        torus { 2, 1 sturm              // bottom arch
            rotate < 90, 0, 0>
            translate < 6, 5, 0>
        }
        clipped_by {
            box { <-1, 5, -1>, <9, 8, 1> }
        }
    }//end union
    

}   // end union `m


#declare Font_Round_n = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <4,0,0> 1         // B/r
    }   // end sphere

    sphere { <0, 7, 0>, 1 }     // T/L

    cylinder { <0,0,0>, <0, 7, 0>, 1     // T/L to B/R stroke
    }
    
    cylinder { <4,0,0>, <4, 5, 0>, 1     // T/L to B/R stroke
    }
    
    torus { 2, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 2, 5, 0>
        clipped_by { box {<-1, 5, -1>, <7, 8, 1> }}
    }

}   // end union `n



#declare Font_Round_o = union {
    torus { 
        3, 1 sturm             
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <-1, -1, -1>, <7, 3, 1> }
        }
    }
    cylinder { <0, 3, 0>, <0, 4, 0>, 1 }

    cylinder { <6, 3, 0>, <6, 4, 0>, 1 }

    torus { 
        3, 1 sturm             
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <-1, 4, -1>, <7, 8, 1> }
        }
    }
}   // end 'o



#declare Font_Round_p = union {
    sphere { <0,13,0> 1              // t/l
    }   // end sphere

    sphere { <0,0,0> 1              // b/l
    }   // end sphere
    
    cylinder { <0, 0, 0>, <0, 13, 0>, 1      //bottom beam
    }

    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 9, 0>
        clipped_by {
            box { <-1, 5, -1>, <7, 9, 1> } 
        }
    }
    
    cylinder { <6, 9, 0>, <6, 10, 0>, 1 };

    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 10, 0>
        clipped_by {
            box { <-1, 10, -1>, <7, 14, 1> } 
        }
    }

}   // end union `p



#declare Font_Round_q = union {
    sphere { <6,13,0> 1              // t/l
    }   // end sphere

    sphere { <6,0,0> 1              // b/l
    }   // end sphere
    
    cylinder { <6, 0, 0>, <6, 13, 0>, 1      //bottom beam
    }

    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 9, 0>
        clipped_by {
            box { <-1, 5, -1>, <7, 9, 1> } 
        }
    }

    cylinder { <0, 9, 0>, <0, 10, 0>, 1 };

    torus { 3, 1 sturm              // top arch
        rotate < 90, 0, 0>
        translate < 3, 10, 0>
        clipped_by {
            box { <-1, 10, -1>, <7, 14, 1> } 
        }
    }


}   // end union `q



#declare Font_Round_r = union {
    sphere { <5,6,0>, 1 }   // end sphere
    sphere { <0,0,0>, 1 }
    sphere { <0,8,0>, 1 }

    cylinder { <0,0,0>, <0,8,0>, 1 }


    torus { 3, 1 sturm              // top left arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <-1, 4, -1>, <3, 8, 1> } 
        }
    }
    torus { 3, 1 sturm              // top right arch
        rotate < 90, 0, 0>
        translate < 3, 4, 0>
        clipped_by {
            box { <-1.5, -1.5, -1>, <1.5, 1.5, 1>
            rotate <0,0,45>
            translate <4,7,0>
            }
        }
    }

}   // end union `R


#declare Font_Round_s = union {
    sphere { <0,3,0> 1           // bottom left end cap
    }   // end sphere
    
    sphere { <6,9,0> 1           // top right end cap
    }   // end sphere
    
    torus { 3, 1 sturm              // top arch   top right
        rotate < 90, 0, 0>
        translate < 3, 9, 0>
        clipped_by {
            box { <-1, 9, -1>, <7, 14, 1> } 
        }
    }
    torus { 3, 1 sturm              // top arch   top right
        rotate < 90, 0, 0>
        translate < 3, 9, 0>
        clipped_by {
            box { <-1, 5, -1>, <3, 14, 1> } 
        }
    }
        
    torus { 3, 1 sturm              // bottom arch    bot/left
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <-1, -1, -1>, <8, 3, 1> }
        }
    }   // end torus
    torus { 3, 1 sturm              // bottom arch    bot/left
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <3, -1, -1>, <8, 7, 1> }
        }
    }   // end torus

}   // end union `s


#declare Font_Round_t = union {
    
    sphere { <0,7.5,0> 1         // T/L
    }   // end sphere

    sphere { <6,7.5,0> 1         // T/R
    }   // end sphere

    sphere { <3, 0, 0> 1         // B/C
    }   // end sphere
    sphere { <3, 13, 0> 1         // T/C
    }   // end sphere

    cylinder { <3,0,0>, <3,13,0>, 1     // I beam (center)
    }

    cylinder { <0, 7.5, 0>, <6, 7.5, 0>, 1      // top beam
    }


}   // end union `t



#declare Font_Round_u = union {
    sphere { <0,7,0> 1         // T/L
    }   // end sphere
    
    sphere { <6,7,0> 1         // T/R
    }   // end sphere

    cylinder { <6,3,0>, <6,7,0>, 1     // right side
    }

    cylinder { <0,3,0>, <0,7,0>, 1     // left side
    }

    torus { 3, 1 sturm              // bottom arch
        rotate < 90, 0, 0>
        translate < 3, 3, 0>
        clipped_by {
            box { <-1, -1, -1>, <7, 3, 1> }
        }
    }   // end torus

}   // end union `u


#declare Font_Round_v = union {
    
    sphere { <0,7,0> 1         // T/L
    }   // end sphere

    sphere { <6,7,0> 1         // T/R
    }   // end sphere

    sphere { <3, 0, 0> 1         // B/C
    }   // end sphere

    cylinder { <3,0,0>, <0,7,0>, 1     // left side
    }

    cylinder { <3,0,0>, <6,7,0>, 1     // right side
    }


}   // end union `v



#declare Font_Round_w = union {
    sphere { <0,7,0> 1          // B/L
    }   // end sphere
    
    sphere { <4,7,0> 1         // B/C
    }   // end sphere

    sphere { <8,7,0> 1          // B/R
    }   // end sphere
    cylinder { <0,0,0>, <0, 7, 0>, 1     // T/L to B/R stroke
    }
    cylinder { <4,0,0>, <4, 7, 0>, 1     // T/L to B/R stroke
    }
    cylinder { <8,0,0>, <8, 7, 0>, 1     // T/L to B/R stroke
    }
    union {
        torus { 2, 1 sturm              // bottom arch
            rotate < 90, 0, 0>
            translate < 2, 1.5, 0>
        }
        torus { 2, 1 sturm              // bottom arch
            rotate < 90, 0, 0>
            translate < 6, 1.5, 0>
        }
        clipped_by {
            box { <-1, -1, -1>, <9, 1.5, 1> }
        }
    }//end union
    


}   // end union `w



#declare Font_Round_x = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,7,0> 1         // T/L
    }   // end sphere

    sphere { <6,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <6,7,0> 1         // T/R
    }   // end sphere

    cylinder { <0,0,0>, <6,7,0>, 1     // bl to ur
    }

    cylinder { <6,0,0>, <0,7,0>, 1      // br to ul
    }
}   // end union `x


#declare Font_Round_y = union {
    sphere { <0,13,0> 1          // B/L
    }   // end sphere
    
    sphere { <6,13,0> 1         // T/L
    }   // end sphere

    sphere { <0,0,0> 1          // B/C
    }   // end sphere
    
    cylinder { <0,0,0>, <6, 13, 0>, 1     // ul to c/c stroke
    }

    cylinder { <0,13,0>, <3.5, 7, 0>, 1     // ul to c/c stroke
    }

}   // end union `y



#declare Font_Round_z = union {
    sphere { <0,0,0> 1          // B/L
    }   // end sphere
    
    sphere { <0,7,0> 1         // T/L
    }   // end sphere

    sphere { <6,0,0> 1          // B/R
    }   // end sphere
    
    sphere { <6,7,0> 1         // T/R
    }   // end sphere

    cylinder { <6,7,0>, <0, 0, 0>, 1     // T/L to B/R stroke
    }

    cylinder { <0,7,0>, <6,7,0>, 1      // top side
    }

    cylinder { <0,0,0>, <6,0,0>, 1      // bottom side
    }

}   // end union `z

// end Font_Round Lower Case letters
#version Font_Round_Ver
