/* On the net, somebody asked for a utility to make a brick wall - with a
** hole. Caveat: No brick should be cut, i.e. the hole should appear jagged.
** Thus no simple 'difference' will do.
**
** In order to build up a nice wall, we need to know the dimensions of the
** brick, the starting point, the end point, and the thickness of the wall.
** We also need to know the particulars of the intersecting hole (i.e. 
** centre and radius)
** I think I'll just choose a mortar thinkness of 1/5 of the brick height
** for now.
*/
brick:-
   daso,clss,
   nso(camera([location(20,40,-130), right(-1.33,0,0),look_at(0,-1,0)])),
   nso(light_source([position(20,25,-130),colour(rgb(1,1,1))])),
   nso(light_source([position(-50,100,-100),colour(rgb(1,1,1))])),
   brick_wall([10,3,5],[-80,-32,-5],[-3,32,5],12,[-43,1,0],23.0,circle),
   brick_wall([10,3,5],[3,-32,-5],[80,32,5],12,[43,-2,8],23.0,sphere),
   save('bricks.pov').
brick_wall([BX,BY,BZ],[SX,SY,SZ],[EX,EY,EZ],Thickness,[_CX,_CY,_CZ],Rad,_Mode) :-
   ( BX > EX-SX ; BY > EY-SY ; BZ > EZ-SZ
   ; Thickness<1
   ; BX < 0 ; BY < 0 ; BZ < 0
   ; Rad < 0   %% Should be <=, but by specifiying an empty hole, the user
               %% can define a brick wall, for which there is no other
               %% procedure. ('empty hole'??? What kind of English is that?)
   ),
   !,
   writef('IUIQ error in brick_wall\n').
brick_wall([BX,BY,BZ],[SX,SY,SZ],[EX,EY,_EZ],Thickness,CPoint,Rad,Mode) :-
   MaxX is (EX-SX) // BX,
   MaxY is (EY-SY) // BY,
   Mortar is BY / 5.0,
   X3 is SX+MaxX*BX,
   forall( between(1,Thickness,I), (
      Z1 is SZ+(I-1)*(BZ+Mortar),
      Z2 is Z1+BZ,
      nl,
      forall( between(1,MaxY,J), (
         write('.'),flush,
         %Y1 is SY+(J-1)*(BY+Mortar),
         Y1 is SY+(J-1)*BY+Mortar,
         Y2 is SY+J*BY,
         SX1 is SX - ((I+J) mod 2)*BX / 2.0 + Mortar,
         SX2 is SX + (BX + ((I+J+1) mod 2)*BX) / 2.0,
         (((Mode=circle,
            box_circle_intersection([SX,Y1,Z1],[SX2,Y2,Z2],CPoint,Rad) )
         ;( Mode=sphere, 
            box_sphere_intersection([SX,Y1,Z1],[SX2,Y2,Z2],CPoint,Rad) ) )
         -> true
         ;  accumulate(box([bc(SX,Y1,Z1),tc(SX2,Y2,Z2)]))
         ),
         forall( between(1,MaxX,K), (
            X1 is SX1+BX*K,
            X2 is SX1+BX*(K+1)-Mortar,
            (  K=MaxX
            -> X4=X3
            ;  X4=X2
            ),
            (((Mode=circle,
               box_circle_intersection([X1,Y1,Z1],[X4,Y2,Z2],CPoint,Rad) )
            ;( Mode=sphere, 
               box_sphere_intersection([X1,Y1,Z1],[X4,Y2,Z2],CPoint,Rad) ) )
            -> true
            ;  accumulate(box([bc(X1,Y1,Z1),tc(X4,Y2,Z2)]))
            )
         ))
      ))
   )),
   get_and_clear_accumulator(Bricks),
   nso(object([union(Bricks),texture([
                                 pigment([colour(rgb(0.8,0.3,0.3))]),
                                 finish([crand(0.4)])
                             ])
             ])
   ),
   true.
box_circle_intersection([Bx,By,_Bz],[Tx,Ty,_Tz],[Cx,Cy,_Cz],Rad) :-
   X is (Tx + Bx)/2.0 - Cx,
   Y is (Ty + By)/2.0 - Cy,
   Rad > sqrt(X*X+Y*Y).
box_sphere_intersection([Bx,By,Bz],[Tx,Ty,Tz],[Cx,Cy,Cz],Rad) :-
   X is (Tx + Bx)/2.0 - Cx,
   Y is (Ty + By)/2.0 - Cy,
   Z is (Tz + Bz)/2.0 - Cz,
   Rad > sqrt(X*X+Y*Y+Z*Z).
point_inside_circle([X1,Y1,_Z1],[Cx,Cy,_Cz],Rad) :-
   Dx is X1-Cx,
   Dy is Y1-Cy,
   Rad > sqrt(Dx*Dx+Dy*Dy).
point_inside_sphere([X1,Y1,Z1],[Cx,Cy,Cz],Rad) :-
   Dx is X1-Cx,
   Dy is Y1-Cy,
   Dz is Z1-Cz,
   Rad > sqrt(Dx*Dx+Dy*Dy+Dz*Dz).
