% :- new_screen_object(camera([location(20,20,20),look_at(0,0,0)])).

basic_test :-
   clss, daso, % clear screens and delete all screen objects
   new_screen_object(camera([location(0,9,70),look_at(0,0,0)])),
   new_screen_object(light_source([position(10,10,10),colour(rgb(1,1,1))])),
   new_screen_object(sphere([centre(10,0,10),radius(5),
                             texture(pigment(colour(rgb(1,1,1))))
                     ])),
   new_screen_object(box([tc(4,4,3),bc(20,20,19),
                             texture(pigment(colour(rgb(1,1,1)))),
                             rotate(10,20,30)
                     ])),
   new_screen_object(cylinder([start(-40,0,30),end(-20,0,20),radius(5),
                             texture(pigment(colour(rgb(1,1,1))))
                     ])),
   new_screen_object(cone([start(40,0,-30),radius1(1),end(-20,0,20),radius2(5),
                             texture(pigment(colour(rgb(1,1,1))))
                     ])),
   new_screen_object(union([
                        cylinder([start(2,40,30),end(20,60,20),radius(3)]),
                        box([tc(4,40,3),bc(20,60,19)])
                     ])),
   % add more once they can be displayed...
   true.

spline_test :-
   clss,daso,
   % ensure_loaded('b_spline_coeff'),
   % ensure_loaded('catmull_rom_coeff'),
   writef('\nCreating ''bspline.pov'' and ''cr_spline.pov''\n'),
   writef('Please render in the normal way.\n\n'),
   new_screen_object(camera([location(80,80,80),look_at(0,0,0)])),
   new_screen_object(light_source([position( -10, 100, 100),colour(rgb(1,1,1))])),
   %% generate 20 control points.
   bagof( [X,Y,Z],
      I^(  between(1,20,I),
           X is random(100)-50.0,
           Y is random(100)-50.0,
           Z is random(100)-50.0
      ),
      CPs
   ),
   %% make a tiny box at each control point to show what we're doing.
   forall( member(P,CPs), 
      ( [X,Y,Z]=P,
        X1 is X-1, Y1 is Y-1, Z1 is Z-1,
        X2 is X+1, Y2 is Y+1, Z2 is Z+1,
        new_screen_object(box([bc(X1,Y1,Z1),tc(X2,Y2,Z2),
                          texture(pigment(colour(rgb(1,1,1))))]))
      )),
   %% calculate the bspline points. 200 of them
   spline(b_spline,CPs,200,NPs),
   %% create 200 spheres on the bspline curve. Do it the slow way.
   forall( nmember(P,NPs,N), 
      ( [X,Y,Z]=P,
        N2 is N/200.0,
        R is N2,
        G is 1.0 - (N2*N2),
        B is N2,
        new_screen_object(sphere([centre(X,Y,Z),radius(1),
                          texture(pigment(colour(rgb(R,G,B))))]))
      )),
   save_povray2('bspline.pov'),
   daso,clss,
   new_screen_object(camera([location(80,80,80),look_at(0,0,0)])),
   new_screen_object(light_source([position( -10, 100, 100),colour(rgb(1,1,1))])),
   %% make a tiny box at each control point to show what we're doing.
   forall( member(P,CPs), 
      ( [X,Y,Z]=P,
        X1 is X-1, Y1 is Y-1, Z1 is Z-1,
        X2 is X+1, Y2 is Y+1, Z2 is Z+1,
        new_screen_object(box([bc(X1,Y1,Z1),tc(X2,Y2,Z2),
                          texture(pigment(colour(rgb(1,1,1))))]))
      )),
   %% calculate the catmull_rom spline points. 200 of them
   spline(cr_spline,CPs,200,NP2s),
   %% create 200 spheres on the bspline curve. Do it the slow way.
   forall( nmember(P,NP2s,N), 
      ( [X,Y,Z]=P,
        N2 is N/200.0,
        R is N2,
        G is 1.0 - (N2*N2),
        B is N2,
        new_screen_object(sphere([centre(X,Y,Z),radius(1),
                          texture(pigment(colour(rgb(R,G,B))))]))
      )),
   save_povray2('cr_spline.pov'),
   true.

test :-
   new(P,point(10,20)),
   new(@demo, dialog('Demo Window')),
   send(P, x, 15),
   send(@demo, append, text_item(name)),
   send(@demo,open).

mt :- moebius_test.
moebius_test :-
   clss, daso, % ready the system
   % new_declared(my_texture,texture,texture([pigment(agate)])),
   new_screen_object(camera([location(0,9,70),look_at(0,0,0)])),
   new_screen_object(light_source([position( 100,   0,  30),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(   0, 100,  20),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(-100, -50, 100),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(   0,   9,  90),colour(rgb(1,1,1))])),
   N = 20,
   forall( between(1,N,T), (
      moebius(T, N, [X1,Y1,Z1]),
      moebius(T+N, N, [X2,Y2,Z2]),
      accumulate(cylinder([start(X1,Y1,Z1),end(X2,Y2,Z2),radius(0.5),
                         texture(pigment('Jade'))]))
   )),
   get_and_clear_accumulator(Spheres),
   new_screen_object(union(Spheres)).
   % forall( between(0,N,T), (
      % X is 20*degs_sin(720*T/N),
      % Y is +20*degs_sin(1440*T/N) + 4*degs_cos(360*T/N),
      % Z is -10*degs_cos(720*T/N) - 4*degs_sin(360*T/N),
      % R is 0.5+T/(2*N),
      % G is 0.25+T/(2*N),
      % B is 1.0-T/(2*N),
      % accumulate(sphere([centre(X,Y,Z),radius(1),
                         % texture(pigment(colour(rgb(R,G,B))))]))
   % )),
   % get_and_clear_accumulator(Spheres),
   % new_screen_object(union(Spheres)).
moebius(T, N, [X, Y, Z]) :-
   X is +10*degs_sin(720*T/N) + 4*degs_cos(180*T/N),
   Y is 20*degs_sin(360*T/N),
   Z is +10*degs_cos(360*T/N) - 4*degs_sin(180*T/N).
%   (  T =< N
%   -> X is (3*degs_cos(180*T/N)+20)*degs_cos(360*T/N),
%      Y is 0, % 10*degs_sin(360*T/N),
%      Z is -3*degs_sin(180*T/N)+20*degs_sin(360*T/N)
%   ;  T2 is T-N,
%      X is (3*degs_cos(180*T/N)+20)*degs_cos(360*T2/N),
%      Y is 0, % 10*degs_sin(360*T2/N),
%      Z is (3*degs_sin(180*T/N)+20)*degs_sin(360*T2/N)
%   ).


colour_cubed :-
   delete_all_screen_objects,     % (short form: daso) ; ready the scene.
   new_screen_object(camera([location(20,20,20),look_at(0,0,0)])),
   new_screen_object(light_source([position( 100,0,0),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(0, 100,0),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(0,0, 100),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(10,10,10),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(0,0,0),colour(rgb(1,1,1))])),
   SideX = 8,  % Just to let you know, let's use a var (i.e. const)
   SideY = 8,
   SideZ = 8,
   % three nested 'for' loops...
   forall(  between(0,SideZ,I),   (      % 'for (I=0; I<=SideZ,I++) {
      BottomZ is SideZ - (SideZ-I)*2 -1, %eg SideZ=10 => -11 - +9
      UpperZ is SideZ - (SideZ-I)*2 +1,  %eg SideZ=10 => -9 - +11
      Red is 1.0 - (I/SideZ),
      forall(  between(0,SideY,J),    (
         BottomY is SideY - (SideY-J)*2 -1, 
         UpperY is SideY - (SideY-J)*2 +1,
         Green is 1.0 - (J/SideY),
         forall(  between(0,SideX,K),    (
            BottomX is SideX - (SideX-K)*2 -1, 
            UpperX is SideX - (SideX-K)*2 +1,
            Blue is 1.0 - (K/SideX),
            % save the boxes in a temporary place, to be retrieved later
            accumulate( box([ bc(vector(BottomX,BottomY,BottomZ)),
                              tc(vector(UpperX,UpperY,UpperZ)),
                              texture(pigment(colour(rgb(Red,Green,Blue))))
                           ])
               )
         ))
      ))
   )),
   % retrieve all the saved boxes...
   get_accumulator(Boxes),
   % clean up...
   clear_accumulator,
   % ... the 2 lines above may be written as 'get_and_clear_accumulator(Boxes)'

   % tell the system we've got a new object
   new_screen_object(difference([
                               union(Boxes),
                               cylinder([start(vector(-30,-30,-30)),
                                         end(vector(30,30,30)),
                                         radius(7),
                                         texture(pigment(colour(rgbf(1,1,1,1))))
                                       ])
                            ])).             

cc2 :-
   daso,
   new_screen_object(camera([location(-20,-20,-20),look_at(0,0,0)])), % camera

   % example of how to define White - normally found in colours.pl
   new_declared('White',colour(rgb(1,1,1))),

   new_screen_object(light_source([position( 100,0,0),colour('White')])),
   new_screen_object(light_source([position(-100,0,0),colour('White')])),
   new_screen_object(light_source([position(0, 100,0),colour('White')])),
   new_screen_object(light_source([position(0,-100,0),colour('White')])),
   new_screen_object(light_source([position(0,0, 100),colour('White')])),
   new_screen_object(light_source([position(0,0,-100),colour('White')])),
   % let's include a light-source without 'White', too.
   new_screen_object(light_source([position(20,20,20),colour(rgb(1,1,1))])),

   SideX = 8,  % Just to let you know, let's use a var (i.e. const)
   SideY = 8,
   SideZ = 8,
   % consider the following lines as 3 nested 'for' loops.
   forall(  between(0,SideZ,I),   (
      Z is SideZ - (SideZ-I)*2,
      Red is 1.0 - (I/SideZ),
      forall(  between(0,SideY,J),    (
         Y is SideY - (SideY-J)*2, 
         Green is 1.0 - (J/SideY),
         forall(  between(0,SideX,K),    (
            X is SideX - (SideX-K)*2, 
            Blue is 1.0 - (K/SideX),
            (  (((I=0 ; I=SideZ), ((J=0 ; J=SideY) ; (K=0 ; K=SideX)))
               ;((J=0 ; J=SideY), ((I=0 ; I=SideZ) ; (K=0 ; K=SideX)))
               ;((K=0 ; K=SideX), ((I=0 ; I=SideZ) ; (J=0 ; J=SideY))))
            ->( (I=0,J=0,K=0) -> true ; 
               new_screen_object(
                           sphere([ centre(vector(X,Y,Z)),
                                 radius(1),
                                 texture([pigment(colour(rgb(Red,Green,Blue))),
                                          finish(reflection(0.5))])
                              ])
               ))
            ;  true
            )
         ))
      ))
   )).

colour_cube :-
   daso,clss,
   new_screen_object(light_source([position( 100,0,0),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(-100,0,0),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(0, 100,0),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(0,-100,0),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(0,0, 100),colour(rgb(1,1,1))])),
   new_screen_object(light_source([position(0,0,-100),colour(rgb(1,1,1))])),
   SideX = 5,  % Just to let you know, let's use a var (i.e. const)
   SideY = 5,
   SideZ = 5,
   forall(  between(1,SideZ,I),   (
      BottomZ is SideZ - (SideZ-I)*2 -2, %eg SideZ=10 => -11 - +9
      UpperZ is SideZ - (SideZ-I)*2, %eg SideZ=10 => -9 - +11
      Red is 1.0 - (I/SideZ),
      forall(  between(1,SideY,J),    (
         BottomY is SideY - (SideY-J)*2 -2, 
         UpperY is SideY - (SideY-J)*2,
         Green is 1.0 - (J/SideY),
         forall(  between(1,SideX,K),    (
            BottomX is SideX - (SideX-K)*2 -2, 
            UpperX is SideX - (SideX-K)*2,
            Blue is 1.0 - (K/SideX),
            new_screen_object(
                           box([ bc(vector(BottomX,BottomY,BottomZ)),
                                 tc(vector(UpperX,UpperY,UpperZ)),
                                 texture(pigment(colour(rgb(Red,Green,Blue))))
                              ])
               )
         ))
      ))
   )).

jade_ring :-
   daso, clss,
   new_declared('White',colour(rgb(1,1,1))),
   new_declared(camera_pos,vector(200,0,80)),
   new_screen_object(camera([location(camera_pos),
                             sky(0,0.5,1),
                             look_at(0,0,-15)])), % camera - or had u guessed?
   new_screen_object(light_source([position(camera_pos),colour('White')])),
   new_screen_object(light_source([position(200,0,0),colour('White')])),
   new_screen_object(light_source([position(0,200,0),colour('White')])),
   new_screen_object(light_source([position(0,0,200),colour('White')])),
   new_declared(ring_sphere,object,sphere([centre(0,0,0),radius(6)])),
   new_declared(my_texture,texture,texture([
                                      pigment(['Jade',scale(2,3,4)]),
                                      finish([phong(0.7),phong_size(70)])])),
   NoCircles is 3333*3,
   Size1 = 15,
   Size2 = 100,
   forall(  between(0,NoCircles,I),
      (  Phi1 is 3.0*360*I / NoCircles,
         Phi3 is 34*360*I / NoCircles,
         Z is Size1*degs_sin(Phi3),
         Y2 is Size2+0.5*Size1*degs_cos(Phi3),
         X is Y2*degs_sin(Phi1),
         Y is Y2*degs_cos(Phi1),
         new_screen_object(ring_sphere([translate(X,Y,Z),texture(my_texture)]))
      )
   ).

metal_ring :-
   daso, clss,
   new_declared('White',colour(rgb(1,1,1))),
   %% dummy definitions for the next 4 declarations. Until I get texture.inc
   %% parsed properly.
   new_declared('Jade',texture([pigment([colour(rgb(1,1,1))])])),
   new_declared('Gold_Texture',texture([pigment([colour(rgb(1,1,1))])])),
   new_declared('Silver_Texture',texture([pigment([colour(rgb(1,1,1))])])),
   new_declared('Copper_Texture',texture([pigment([colour(rgb(1,1,1))])])),
   new_declared(camera_pos,vector(200,0,80)),
   new_declared(ring_sphere,object,sphere([centre(0,0,0),radius(6)])),
   new_screen_object(camera([location(camera_pos),
                             sky(0,0.5,1),
                             look_at(0,0,-15)])), % camera - or had u guessed?
   new_screen_object(light_source([position(camera_pos),colour('White')])),
                   % hm. isn't that -^ rather much like ambient 0.2 (say)?
   new_screen_object(light_source([position(200,0,0),colour('White')])),
   new_screen_object(light_source([position(0,200,0),colour('White')])),
   new_screen_object(light_source([position(0,0,200),colour('White')])),
   new_screen_object(torus([major_radius(100),minor_radius(4.5),
                     scale(1,2,1),
                     rotate(90,0,0),
                     texture([pigment([colour('Jade'),
                                       scale(6,7,9),
                                       rotate(-10,20,30)
                                     ])
                            ])
                    ])),
   NoCircles is 3333,
   Size1 = 15,
   Size2 = 100,
   !,
   forall(  between(0,NoCircles,I),
      (  Phi1 is 360*I / NoCircles,
         Phi3 is 11*360*I / NoCircles,
         Y is Size2+0.5*Size1*degs_cos(Phi3),
         X1 is Y*degs_sin(Phi1),
         Y1 is Y*degs_cos(Phi1),
         X2 is Y*degs_sin(Phi1+120),
         Y2 is Y*degs_cos(Phi1+120),
         X3 is Y*degs_sin(Phi1+240),
         Y3 is Y*degs_cos(Phi1+240),
         Z is Size1*degs_sin(Phi3),
         new_screen_object(ring_sphere([translate(X1,Y1,Z),texture('Gold_Texture')])),
         new_screen_object(ring_sphere([translate(X2,Y2,Z),texture('Silver_Texture')])),
         new_screen_object(ring_sphere([translate(X3,Y3,Z),texture('Copper_Texture')]))
      )
   ).

plaited_ring :-  %%% Not finished - do not use!
   daso, clss,
   new_declared('White',colour(rgb(1,1,1))),
   %% dummy definitions for the next 4 declarations. Until I get texture.inc
   %% parsed properly.
   new_declared('Jade',texture([pigment([colour(rgb(1,1,1))])])),
   new_declared('Gold_Texture',texture([pigment([colour(rgb(1,1,1))])])),
   new_declared('Silver_Texture',texture([pigment([colour(rgb(1,1,1))])])),
   new_declared('Copper_Texture',texture([pigment([colour(rgb(1,1,1))])])),
   new_declared(camera_pos,vector(200,0,80)),
   new_declared(ring_sphere,object,sphere([centre(0,0,0),radius(6)])),
   new_screen_object(camera([location(camera_pos),
                             sky(0,0.5,1),
                             look_at(0,0,-15)])), % camera - or had u guessed?
   new_screen_object(light_source([position(200,0,0),colour('White')])),
   new_screen_object(light_source([position(0,200,0),colour('White')])),
   new_screen_object(light_source([position(0,0,200),colour('White')])),
   NoCircles is 333,
   Dots is NoCircles//79,
   Size1 = 15,
   Size2 = 100,
   !,
   % forall(  between(0,NoCircles,I),
   forall(  between(0,100,I),   %% only do a bit
      (  (  0 is I mod Dots -> write('.'),nl ; true),
         Phi1 is 360*I / NoCircles,
         Phi3 is 12*360*I / NoCircles,
         Phi2 is Phi3/2,
         Dsin is degs_sin(Phi1),
         Dcos is degs_cos(Phi1),
         D1 is Size2+Size1*degs_sin(Phi2),
         X1 is D1*Dsin,
         Y1 is D1*Dcos,
         Z1 is Size1*degs_cos(Phi3),
         D2 is Size2+Size1*degs_sin(Phi2+120),
         X2 is D2*Dsin,
         Y2 is D2*Dcos,
         Z2 is Size1*degs_cos(Phi3+120),
         D3 is Size2+Size1*degs_sin(Phi2+240),
         X3 is D3*Dsin,
         Y3 is D3*Dcos,
         Z3 is Size1*degs_cos(Phi3+240),
         %new_screen_object(ring_sphere([translate(X1,Y1,Z),texture('Gold_Texture')])),
         %new_screen_object(ring_sphere([translate(X2,Y2,Z),texture('Silver_Texture')])),
         %new_screen_object(ring_sphere([translate(X3,Y3,Z),texture('Copper_Texture')]))
         new_screen_object(ring_sphere([translate(X1,Y1,Z1),texture([colour(rgb(1,0,0))])])),
         new_screen_object(ring_sphere([translate(X2,Y2,Z2),texture([colour(rgb(0,1,0))])])),
         new_screen_object(ring_sphere([translate(X3,Y3,Z3),texture([colour(rgb(0,0,1))])]))
      )
   ).

t1 :-
   new_declared(my_sphere,object,sphere([centre(1,2,3),radius(1),transformations([translate(-20,-20,-20)])])),
   (  between(1,20,I),
      J is I*18, degs_to_rads(J,R), X is 20*sin(R)+40, Y is 40*cos(R)+60,
      new_screen_object(my_sphere([scale(X,Y,I),translate(20,40,50)])),
      fail
   ;  true
   ).

t2 :-
   new_screen_object(sphere([centre(220,100,-300),radius(200)])),
   new_declared(my_sphere,object,sphere([centre(0,0,0),radius(7)])),
   (  between(1,20,I),
      D1 is I*1.5, degs_to_rads(D1,R1),
      D2 is I*2, degs_to_rads(D2,R2),
      D3 is I*2.5, degs_to_rads(D3,R3),
      OD is I*0.5, degs_to_rads(OD,O),
      WD is I/36.0, degs_to_rads(WD, Whee),
      X is 75*cos(R1)+20*cos(Whee)+15*cos(O),
      Y is 75*cos(R2),
      Z is 75*sin(R3)+15*sin(O),
%      new_screen_object(my_box([translate(X,Y,Z)])),
      new_screen_object(my_sphere([translate(X,Y,Z)])),
      fail
   ;  true
   ).

