/*
Personally, I think that a 'balloon-tyre' looks silly and unrealistic. I
have therefore avoided the use of a torus as the base in my example tyre.
The 'underside' of the tyre is part of a sphere, the sides are parts of
tori which intersect this sphere.
Finally, there's a cylinder to cut away the hub parts we don't want.
Thus, the tyre should (hopefully?) look a bit like this:
   ._-_.
  /     \
  |     |
  \_____/


   _____
  /     \
  |     |
  \     /
   `-_-'    (The intelligent reader may have guessed that I don't really
             like ASCII... Not that I prefer EBCDIC, of course.)

The tyre has a herringbone pattern, as opposed to 'V'. This is because I've
never seen a 'V' tyre in my life, whereas I see those `, patterns almost
every day.                                            `,
If you insist on 'V', change Angle2 is (360.0/Threads)*(I-0.5),
to                           Angle2 is (360.0/Threads)*I,

Share and Enjoy?
*/

%% Declare a tyre with Major and Minor radi, with Threads threads, which
%% have a depth of ThrDepth
%% E.g. tyre(0.50, 0.1, 20, 0.03) will create a tyre with a radius of 0.63
%% with 20 threads, which, as it happens, looks quite good. 
%% The radius of the hole in the middle would be: Maj-Min-ThrDepth=0.37
tyre(Maj,Min,Threads,ThrDepth) :-
   BaseRad is Maj+Min-ThrDepth,
   MaxRad is Maj+Min,
   MaxRad2 is MaxRad*1.1,
   Maj21 is Maj*0.9,
   Maj22 is (Maj-Min)*0.9,
   Maj3 is BaseRad-2*Min,
   forall( between(1,Threads,I), (
      Angle1 is (360.0/Threads)*I,
      Angle2 is (360.0/Threads)*(I-0.5),
      YBox is Min*2,     %% should really be 1.414..., but it'll get cut...
      accumulate(box([bc(0,0,0),tc(MaxRad2,YBox,ThrDepth),
                  rotate(45,Angle1,0)
               ])
      ),
      accumulate(box([bc(0,-YBox,0),tc(MaxRad2,0,ThrDepth),
                  rotate(-45,Angle2,0)
               ])
      )
   )),
   get_and_clear_accumulator(Boxes),
   declare('Tyre',
      object([
         intersection([
            union([
               sphere([centre(0,0,0),radius(BaseRad)]),
               intersection([
                  sphere([centre(0,0,0),radius(MaxRad)]),
                  union(Boxes)
               ])
            ]),
            cylinder([start(0,-Maj,0),end(0.001,Maj,0),radius(Maj3),inverse]),
            torus([maj_rad(Maj21),min_rad(Maj),translate(0,Maj22,0)]),
            torus([maj_rad(Maj21),min_rad(Maj),translate(0,-Maj22,0)])
         ]),
         texture(['Tyre_texture']),
         bounded_by([
            cylinder([start(0,-2*Min,0),end(0,2*Min,0),radius(MaxRad)])
         ])
      ])
   ),
   true.

tyre_support :-
   include_file(none),
   daso,
   camera([location(2,1,2.7),look_at(0,0,0)]),
   light_source([position(20,20,20),colour(rgb(1,1,1))]),
   set_option(minor_edges,15),
   set_option(major_edges,10),
   nso('Tyre').
