:- discontiguous display_object/1.

% display_objects :-
%    (  bagof(O, Dud^screen_object(Dud,O), Os)
%    -> display_objects(Os)
%    ;  true
%    ).
display_objects :-
   (  bagof(O, recorded(source,O), Os)
   -> display_objects(Os)
   ;  true
   ).
display_objects([]).
display_objects([H|T]) :-
   display_object(H),
   display_objects(T).

get_transformations([],[]) :-!.
get_transformations([rotate(A,B,C)|T],[rotate(A,B,C)|TR]) :-
   !,
   get_transformations(T,TR).
get_transformations([translate(A,B,C)|T],[translate(A,B,C)|TR]) :-
   !,
   get_transformations(T,TR).
get_transformations([scale(A,B,C)|T],[scale(A,B,C)|TR]) :-
   !,
   get_transformations(T,TR).
get_transformations([_|T],TR) :-
   get_transformations(T,TR).

% box *************************
display_object(box(A)) :-
   ( member(bc(vector(LX,LY,LZ)),A) ; member(bc(LX,LY,LZ),A)),
   ( member(tc(vector(TX,TY,TZ)),A) ; member(tc(TX,TY,TZ),A)),
   get_transformations(A,Trans),
   !,
   Points = [ point(LX,LY,LZ),
              point(TX,LY,LZ),
              point(TX,TY,LZ),
              point(LX,TY,LZ),
              point(LX,LY,TZ),
              point(TX,LY,TZ),
              point(TX,TY,TZ),
              point(LX,TY,TZ)],
   transform_points(Trans,Points,[point(X1,Y1,Z1), point(X2,Y2,Z2),
                                  point(X3,Y3,Z3), point(X4,Y4,Z4),
                                  point(X5,Y5,Z5), point(X6,Y6,Z6),
                                  point(X7,Y7,Z7), point(X8,Y8,Z8)]),
   draw_lines([ line(point(X1,Y1,Z1),point(X2,Y2,Z2)),
                line(point(X1,Y1,Z1),point(X4,Y4,Z4)),
                line(point(X1,Y1,Z1),point(X5,Y5,Z5)),

                line(point(X3,Y3,Z3),point(X2,Y2,Z2)),
                line(point(X3,Y3,Z3),point(X4,Y4,Z4)),
                line(point(X3,Y3,Z3),point(X7,Y7,Z7)),

                line(point(X6,Y6,Z6),point(X2,Y2,Z2)),
                line(point(X6,Y6,Z6),point(X5,Y5,Z5)),
                line(point(X6,Y6,Z6),point(X7,Y7,Z7)),

                line(point(X8,Y8,Z8),point(X4,Y4,Z4)),
                line(point(X8,Y8,Z8),point(X5,Y5,Z5)),
                line(point(X8,Y8,Z8),point(X7,Y7,Z7))
              ]),
   (  option(cube_faces,true)
   -> draw_lines([ line(point(X1,Y1,Z1),point(X3,Y3,Z3)),
                   line(point(X1,Y1,Z1),point(X6,Y6,Z6)),
                   line(point(X1,Y1,Z1),point(X8,Y8,Z8)),
                   line(point(X2,Y2,Z2),point(X4,Y4,Z4)),
                   line(point(X2,Y2,Z2),point(X5,Y5,Z5)),
                   line(point(X2,Y2,Z2),point(X7,Y7,Z7)),
                   line(point(X5,Y5,Z5),point(X4,Y4,Z4)),
                   line(point(X5,Y5,Z5),point(X7,Y7,Z7)),
                   line(point(X6,Y6,Z6),point(X4,Y4,Z4)),
                   line(point(X7,Y7,Z7),point(X4,Y4,Z4)),
                   line(point(X8,Y8,Z8),point(X6,Y6,Z6)),
                   line(point(X8,Y8,Z8),point(X4,Y4,Z4))
                 ])
   ;  true
   ).
% box *************************

% sphere **********************
display_object(sphere(A)) :-
   information(display,'display sphere',A),
   ( member(centre(vector(XC,YC,ZC)),A) ; member(centre(XC,YC,ZC),A)),
   member(radius(Radius),A),
   !,
   information(display,'centre,radius',[[XC,YC,ZC],[A]]),
   get_transformations(A,Trans),
   % if you scale (unequally) and _then_ rotate, then I have to draw an ugly sphere
   % (( nmember(rotate(Rx,Ry,Rz),Trans,RN),nmember(scale(Sx,Sy,Sz),Trans,SN),SN<RN )
   (( member(rotate(_,_,_),Trans),member(scale(_,_,_),Trans) )
   -> true % expensive transformations... 'globe' the sphere
   ;  transform_points(Trans,[point(XC,YC,ZC)],[point(Cx,Cy,Cz)]),
      (  bagof([Sx,Sy,Sz], member(scale(Sx,Sy,Sz), Trans), Scales)
      -> wot_name([[Radius,Radius,Radius]|Scales],Radii)
      ;  Radii = radii(Radius, Radius, Radius)
      ),
      draw_sphere(centre(Cx,Cy,Cz),Radii)
   ).
wot_name([],radii(1,1,1)) :- !.
wot_name([[Hx,Hy,Hz]|Tail],radii(Rx,Ry,Rz)) :-
   wot_name(Tail,radii(Tx,Ty,Tz)),
   Rx is Hx*Tx,
   Ry is Hy*Ty,
   Rz is Hz*Tz.
% sphere **********************

% cylinder ********************
display_object(cylinder(A)) :-
   !,
   ( member(start(vector(XS,YS,ZS)),A) ; member(start(XS,YS,ZS),A)),
   ( member(end(vector(XE,YE,ZE)),A) ; member(end(XE,YE,ZE),A)),
   (( XE=XS, YE=YS, ZE=ZS)
   -> write('ERROR!!! null-cylinder - not drawn'),nl
   ;  member(radius(Radius),A),
      get_transformations(A,Trans),
      transform_points(Trans,[point(XS,YS,ZS),point(XE,YE,ZE)],
                             [point(XS2,YS2,ZS2),point(XE2,YE2,ZE2)]),
      (  bagof([Sx,Sy,Sz], member(scale(Sx,Sy,Sz), Trans), Scales)
      -> wot_name([[Radius,Radius,Radius]|Scales],Radii),
         Radii = radii(Radius2,_,_)
      ;  Radius2 = Radius
      ),
      UpVX is XE2-XS2,
      UpVY is YE2-YS2,
      UpVZ is ZE2-ZS2,
      get_circle_points(vector(XS2,YS2,ZS2),
                        vector(UpVX,UpVY,UpVZ),
                        Radius2,
                        PointsStart),
      get_circle_points(vector(XE2,YE2,ZE2),
                        vector(UpVX,UpVY,UpVZ),
                        Radius2,
                        PointsEnd),
      length(PointsEnd,LP),
      forall(  between(1,LP,T),
         (  nmember(P1,PointsStart,T),
            nmember(P2,PointsEnd,  T),
            draw_line(line(P1,P2))
         )
      ),
      (  option(triangulated_cylinder,true)
      -> PointsEnd = [P4|_],
         forall(  between(1,LP,T),
            (  nmember(P1,PointsStart,T),
               T2 is T+1,
               (  nmember(P3,PointsEnd,  T2)
               -> draw_line(line(P1,P3))
               ;  draw_line(line(P1,P4))
               )
            )
         )
      ;  true
      ),
      draw_poly_lines(PointsStart),
      draw_poly_lines(PointsEnd)
   ).
% cylinder ****************

% torus *******************
display_object(torus(A)) :-
   !,
   ( member(major_radius(MajR),A) ; member(maj_rad(MajR),A)),
   ( member(minor_radius(MinR),A) ; member(min_rad(MinR),A)),
   (( MinR > MajR)
   -> write('Warning! Minor radius bigger than Major radius'),nl
   ;  true
   ),
   option(minor_edges,Edgesm),
   option(major_edges,EdgesM),
   get_transformations(A,Trans),
   bagof( point(X,Y,Z),
      I^Degs^(
         between(1,Edgesm,I),
         Degs is I*360/Edgesm,
         X is 0,
         Y is degs_sin(Degs)*MinR,
         Z is degs_cos(Degs)*MinR + MajR
      ),
      CPs
   ),
   bagof( Points,
      I^Degs^Ps^(
         between(1,EdgesM,I),
         Degs is I*360/EdgesM,
         transform_points([rotate(0,Degs,0)],CPs,Ps),
         transform_points(Trans,Ps,Points)
      ),
      Pointss
   ),
   !,
   display_object_torus(Pointss).
display_object_torus_2([P1|Ps]) :-
   !,
   append([P1|Ps],[P1],Pss),
   draw_poly_lines(Pss).
display_object_torus_1([Ps]) :- 
   !,
   display_object_torus_2(Ps).
display_object_torus_1([P1s,P2s|Ps]) :-
   display_object_torus_2(P1s),
   length(P1s,LP),
   forall(  between(1,LP,T),
      (  nmember(P1,P1s,T),
         nmember(P2,P2s,T),
         draw_line(line(P1,P2))
      )
   ),
   display_object_torus_1([P2s|Ps]).
display_object_torus([P1s|Ps]) :-
   append([P1s|Ps],[P1s],Points),
   display_object_torus_1(Points).
% torus *******************

% cone ********************
display_object(cone(A)) :-
   !,
   ( member(start(vector(XS,YS,ZS)),A) ; member(start(XS,YS,ZS),A)),
   ( member(end(vector(XE,YE,ZE)),A) ; member(end(XE,YE,ZE),A)),
   member(radius1(Radius1),A),
   member(radius2(Radius2),A),
   ((( XE=XS, YE=YS, ZE=ZS) ; Radius1=Radius2)
   -> write('ERROR!!! null-cone - not drawn'),nl
   ;  get_transformations(A,Trans),
      transform_points(Trans,[point(XS,YS,ZS),point(XE,YE,ZE)],
                             [point(XS2,YS2,ZS2),point(XE2,YE2,ZE2)]),
      (  bagof([Sx,Sy,Sz], member(scale(Sx,Sy,Sz), Trans), Scales)
      -> wot_name([[Radius1,Radius1,Radius1]|Scales],Radii),
         Radii = radii(Radius12,_,_)
      ;  Radius12 = Radius1
      ),
      (  bagof([Sx,Sy,Sz], member(scale(Sx,Sy,Sz), Trans), Scales)
      -> wot_name([[Radius2,Radius2,Radius2]|Scales],Radii),
         Radii = radii(Radius22,_,_)
      ;  Radius22 = Radius2
      ),
      UpVX is XE2-XS2,
      UpVY is YE2-YS2,
      UpVZ is ZE2-ZS2,
      get_circle_points(vector(XS2,YS2,ZS2),
                        vector(UpVX,UpVY,UpVZ),
                        Radius12,
                        PointsStart),
      get_circle_points(vector(XE2,YE2,ZE2),
                        vector(UpVX,UpVY,UpVZ),
                        Radius22,
                        PointsEnd),
      length(PointsEnd,LP),
      forall(  between(1,LP,T),
         (  nmember(P1,PointsStart,T),
            nmember(P2,PointsEnd,  T),
            draw_line(line(P1,P2))
         )
      ),
      draw_poly_lines(PointsStart),
      draw_poly_lines(PointsEnd)
   ).
% cone ********************

% light_source ****************
display_object(light_source(_A)) :-
   option(display_lightsource,false), !.
display_object(light_source(A)) :-
   !,
   member(position(X,Y,Z),A),
   !,
   option(light_source_size,Size),
   win_scale(xyWindow,ScaleXY),
   win_scale(zyWindow,ScaleZY),
   win_scale(xzWindow,ScaleXZ),
   win_centre(xyWindow,[X_1,Y_1]),
   win_centre(zyWindow,[Z_2,Y_2]),
   win_centre(xzWindow,[X_3,Z_3]),
   get_transformations(A,Trans),
   transform_points(Trans,[point(X,Y,Z)],[point(X1,Y1,Z1)]),

   Xxy0 is X_1+X1*ScaleXY, Yxy0 is Y_1-Y1*ScaleXY,
   send(@xyWindow,display,line(Xxy0-Size, Yxy0, Xxy0+Size, Yxy0)),
   send(@xyWindow,display,line(Xxy0, Yxy0-Size, Xxy0, Yxy0+Size)),

   Zzy0 is Z_2+Z1*ScaleZY, Yzy0 is Y_2-Y1*ScaleZY,
   send(@zyWindow,display,line(Zzy0-Size, Yzy0, Zzy0+Size, Yzy0)),
   send(@zyWindow,display,line(Zzy0, Yzy0-Size, Zzy0, Yzy0+Size)),

   Xxz0 is X_3+X1*ScaleXZ, Zxz0 is Z_3-Z1*ScaleXZ,
   send(@xzWindow,display,line(Xxz0-Size, Zxz0, Xxz0+Size, Zxz0)),
   send(@xzWindow,display,line(Xxz0, Zxz0-Size, Xxz0, Zxz0+Size)).
% light_source ****************

% smooth triangle *************
display_object(smooth_triangle(A)) :-
   !,
   member(vertices(B,_C,D,_E,F,_G),A),
   display_object(triangle([vertices(B,D,F)])).
% smooth triangle *************

% triangle ********************
display_object(triangle(A)) :-
   !,
   member(vertices(V1,V2,V3),A),
   get_vector_components(V1,X11,Y11,Z11),
   get_vector_components(V2,X21,Y21,Z21),
   get_vector_components(V3,X31,Y31,Z31),
   get_transformations(A,Trans),
   Points = [ point(X11,Y11,Z11),
              point(X21,Y21,Z21),
              point(X31,Y31,Z31)],
   transform_points(Trans,Points,[point(X1,Y1,Z1), point(X2,Y2,Z2),
                                  point(X3,Y3,Z3)]),
   draw_lines([ line(point(X1,Y1,Z1),point(X2,Y2,Z2)),
                line(point(X1,Y1,Z1),point(X3,Y3,Z3)),
                line(point(X2,Y2,Z2),point(X3,Y3,Z3))]),
   !.
% triangle ********************

%% Needs more work... cf castle.pl, and the 'top-rocks'
% object **********************
display_object(object([Name|Descr2])) :-
   % A =.. [Name|_Translations_etc],
   % information('Can I display %t?\n',[Name]),
   (  declared(_,Name,_Type,Descr0)
   -> (Descr0 =.. [Type,Descr1] ; Descr0 =.. [Type|Descr1]), !,
      append(Descr1,Descr2,Description),
      information('Type: %t - Description: %t\n',[Type,Description]),
   %% Hm2 - for each member of the union/merge etc, we need to add the
   %% transformations.
   %% Still to be done
      Object =.. [Type|Description],
      display_object(Object)
   ;  forall( member(Obj,[Name|Descr2]),
            ignore(( Obj =.. [Obj2|_],
                     can_display(Obj2),
                     display_object(Obj)
      )))
   ).
% union ***********************
   ).
%true.
%    can_display(Name),
%    writef('Apparently I can!\n'),
%    display_object(Name).
% object **********************

% union ***********************
display_object(Composite) :-
   Composite =.. [CompType,A],
   % write(tutu),nl,
   member(CompType,[union,merge,difference,intersection]),
   !,
   % Hm. A quick hack is needed.
   %% Hm2 - for each member of the union/merge etc, we need to add the
   %% transformations.
   (  member(_,A)
   -> forall( member(Obj,A),
         ignore(( can_display(Obj),
                  display_object(Obj)
         )))
   ;  display_object(A)
   ).
% union ***********************

% declared ********************
% If our object does not match any of the above, it may be a declared object
display_object(NamedObject) :-
   NamedObject =.. [Name|SecAttribs],
   declared(_,Name,_,Object),
   !,  % yes, there is such a declared object!
   Object =.. [ObjectType|Description],
   flatten([Description,SecAttribs],Attribs),
   Object2 =.. [ObjectType,Attribs],
   display_object(Object2).
% declared ********************

% ignore **********************
display_object(NameO) :-
   NameO =.. [Name|_],
   member(Name,[plane,bounded_by,texture,rotate,translate,rotation]),
   !.
% ignore **********************

% something else **************
display_object(H) :-
  message(errorWindow,'I don''t know how to display:'),
  message(errorWindow,H).
% something else **************

% Can I draw this thing wot I've found?
can_display(Obj) :-
   Obj =.. [Type|_],
   member(Type,[box,  % list of objects I know how to display
                sphere,
                triangle,
                smooth_triangle,
                cylinder,
                light_source,
                cone,
                torus,
                union,
                merge,
                plane,          % well, I won't display it, but I don't like
                                % the erroe messages!
                difference,
                intersection,
                object
               ]),
   !.
can_display(Obj) :-
   Obj =.. [Name|_],
   declared(_,Name,Type,_),
   writef('Found %t as a declared of type %t\n',[Name,Type]),
   can_display(Type).
% Well? Could I display it?
% can_display(H) :-
%   message(errorWindow,'I don''t know how to display:'),
%   message(errorWindow,H).

get_circle_points(Centre, Upvector, Radius, Points) :-
   option(edges,Edges),
   bagof( point(X,0,Z), 
      X^Z^T^(
         between(0,Edges,T),
         X is Radius*degs_cos(360*T / Edges),
         Z is Radius*degs_sin(360*T / Edges) 
      ),
      Points_1
   ),
   !,
   (  Centre = vector(CX,CY,CZ) ; Centre = centre(CX,CY,CZ) ; Centre = centre(vector(CX,CY,CZ))),
   vector_angles(Upvector,Alpha2,Theta2),
   % transform_points([rotate(Alpha2,0,0),rotate(0,Theta2,0),translate(CX,CY,CZ)],
   transform_points([rotate(Alpha2,0,Theta2),translate(CX,CY,CZ)],
         Points_1,Points),
   !,
   true. %% RJH write(Points),nl.

get_vector_components(vector(X,Y,Z),X1,Y1,Z1) :-
   X1 = X, Y1 = Y, Z1 = Z.
get_vector_components(vector(V),X1,Y1,Z1) :-
   !,
   declared(_,V,vector,Object),
   get_vector_components(Object,X1,Y1,Z1).
get_vector_components([X,Y,Z],X1,Y1,Z1) :-
   !,
   get_vector_components(vector(X,Y,Z),X1,Y1,Z1).
