% :- discontiguous parse_pass_1_pov2.
:- discontiguous parse_pass_1_pov2_1/4.
:- discontiguous parse_pass_1_pov2_2/4.
:- discontiguous parse_pass_1_pov2_3/4.

parse_file(povray2,Filename) :-
   !,
   flag(reading_file,Old,source),
   % ignore(retract(status(reading_file,_))),
   % assert(status(reading_file,not_included)),
   parse_pov2(Filename),
   flag(reading_file,_,Old).
   % ignore(retract(status(reading_file,_))).

parse_pov2(Filename) :-
   read_pov2(Filename,Contents),
string_to_list(String,Contents),write(String),nl, % delete
   !,
   parse_pov2_2(Contents).

ptest :- parse_file(povray2,'test.pov').
ctest :- parse_file(povray2,'cols.pov').
test2 :- parse_file(povray2,'test2.pov').
test4 :- parse_file(povray2,'test4.pov').

/********************************************************************
** Read from a named file, discarding comments and doublespaces
*/
read_pov2(Filename,Contents) :-
   seeing(Old),
   (  exists_file(Filename)
   -> see(Filename)
   ;  get_pov_include_path(PovPath),
      merge_path_file(PovPath,Filename,RealFile),
      see(RealFile)
   ),
   get(Ch),
   (  read_pov2_2(Ch,Contents) ; true),
   seen,
   see(Old).

read_pov2_2(-1,[]) :- !. % end-of-file
read_pov2_2(White,[32|Chs]) :-
   is_white(White),
   get(A), % if we've got >2 spaces, only save one
   !,      % this we do by using get....
   read_pov2_2(A,Chs).
read_pov2_2(47,Chs) :-
   get0(A),
   (  A = 47   % ...so we have //, and should skip the line
   -> read_line(_),
      get0(B),
      read_pov2_2(B,Chs)
   ;  (  A = 42 % ...so we have /*, and should skip until the end of the comment
      -> read_comment_pov2(1),
         get0(B),
         read_pov2_2(B,Chs)
      ;  read_pov2_2(A,Chs2),
         Chs = [47|Chs2]
      )
   ).
read_pov2_2(C,[C|Chs]) :-
   get0(C2),
   read_pov2_2(C2,Chs).

read_line(Cs) :-
   get0(A),
   read_line_2(A,Cs).
read_line_2(10,[]) :- !.
read_line_2(-1,[]) :- !. % end-of-file
read_line_2(C,[C|Cs]) :-
   get0(C2),
   read_line_2(C2,Cs).

read_comment_pov2(0) :- !.
read_comment_pov2(CommentLevel) :-
   get(A),
   read_comment_pov2_2(A,CommentLevel,CommentLevel2),
   !,
   read_comment_pov2(CommentLevel2).
read_comment_pov2_2(-1,_,_) :- 
   !, 
   write('END OF FILE INSIDE WHAT I THOUGHT WAS A COMMENT!'),nl.
read_comment_pov2_2(47,CL,CL2) :-
   get0(42),  % '/*' - i.e. start of comment.
   !,
   CL2 is CL+1.
read_comment_pov2_2(42,CL,CL2) :-
   get0(47),  % '*/' - i.e. end of comment.
   !,
   CL2 is CL-1.
read_comment_pov2_2(_,CL,CL).
/********************************************************************/

/********************************************************************
** Routines for parsing whatever was got from the input file.
*/
parse_pov2_2(Contents) :-
   parse_pass_1_pov2(Contents, Pass1), % Tokenize
   !,	% ok, we found one valid interpretation. that's enough.
   % write(Pass1),nl,
   parse_pass_2_pov2(Pass1).           % process

parse_pass_1_pov2([], []) :- !. % Terminating condition
parse_pass_1_pov2([32|Rest], Pass1) :- % only spaces left.
   !,
   parse_pass_1_pov2(Rest,Pass1).

% top-level *********************
% Top-level parser. One of:
% fog, background, camera, declare, max_trace_level, max_intersections, default
% (version)
% Of course, #include is a bit special.
parse_pass_1_pov2([35|WordList],ReT) :-
   list_string_match(WordList,include,WL1),
   !,
   append(_,[34|WL2],WL1),
   append(FN,[34|NewWordList],WL2),
   string_to_list(String,FN),
   string_to_atom(String,FileName),
   include_file(FileName),
   !,
   information(parse,'############ include file',FileName),
   parse_include_pov2(FileName),        % later....
   information(parse,'############ end include file',FileName),
   !,
   parse_pass_1_pov2(NewWordList,ReT).    % do rest.
parse_pass_1_pov2(WordList,[max_trace_level(ReH)|ReT]) :-
   list_string_match(WordList,max_trace_level,NewWordList),
   parse_pass_1_pov2_1(parse_max_trace_level,NewWordList,ReH,NewWordList2),
   !,
   information(parse,max_trace_level,ReH),
   parse_pass_1_pov2(NewWordList2,ReT).
parse_pass_1_pov2(WordList,[fog(ReH)|ReT]) :-
   list_string_match(WordList,fog,NewWordList),
   parse_pass_1_pov2_1(parse_fog,NewWordList,ReH,NewWordList2),
   !,
   information(parse,fog,ReH),
   parse_pass_1_pov2(NewWordList2,ReT).
parse_pass_1_pov2(WordList,[background(colour(ReH))|ReT]) :-
   list_string_match(WordList,background,[A|NewWordList]),
   is_white(A),
   !,
   (  list_string_match(NewWordList,colour,NewWordList1)
   ;  list_string_match(NewWordList,color,NewWordList1)),
   parse_pass_1_pov2_1(parse_colour,NewWordList1,ReH,NewWordList2),
   !,
   information(parse,'background colour',ReH),
   parse_pass_1_pov2(NewWordList2,ReT).
parse_pass_1_pov2(WordList,[camera(ReH)|ReT]) :-
   list_string_match(WordList,camera,NewWordList),
   parse_pass_1_pov2_1(parse_camera,NewWordList,ReH,NewWordList2),
   !,
   information(parse,camera,ReH),
   parse_pass_1_pov2(NewWordList2,ReT).
parse_pass_1_pov2([35|WordList],ReT) :-
   list_string_match(WordList,declare,NewWordList),
   parse_pass_1_pov2_1(parse_declare,NewWordList,[String,Wot,Def],NewWordList2),
   !,
   % information(parse,declare,[String,Def]),
   flag(reading_file,Status,Status),
   string_to_atom(String,Name),
   new_declared(Status,Name,Wot,Def),
   parse_pass_1_pov2(NewWordList2,ReT).
parse_pass_1_pov2(WordList,[max_trace_level(ReH)|ReT]) :-
   list_string_match(WordList,max_trace_level,NewWordList),
   parse_pass_1_pov2_1(parse_max_trace_level,NewWordList,ReH,NewWordList2),
   !,
   information(parse,max_trace_level,ReH),
   parse_pass_1_pov2(NewWordList2,ReT).
parse_pass_1_pov2(WordList,[max_intersections(ReH)|ReT]) :-
   list_string_match(WordList,max_intersections,NewWordList),
   parse_pass_1_pov2_1(parse_max_intersections,NewWordList,ReH,NewWordList2),
   !,
   information(parse,max_intersections,ReH),
   parse_pass_1_pov2(NewWordList2,ReT).
parse_pass_1_pov2(WordList,[default(ReH)|ReT]) :-
   list_string_match(WordList,default,NewWordList),
   parse_pass_1_pov2_1(parse_default,NewWordList,ReH,NewWordList2),
   !,
   information(parse,default,ReH),
   parse_pass_1_pov2(NewWordList2,ReT).
parse_pass_1_pov2([35|WordList],ReT) :-
   list_string_match(WordList,version,WordList1),
   !,
   parse_pass_1_pov2_1(parse_float,WordList1,Float,NewWordList),
   information(parse,'version (ignored - only 2+ supported)',Float),
   !,
   parse_pass_1_pov2(NewWordList,ReT).
parse_pass_1_pov2(WordList,[Object|ReT]) :-
   % the others failed - we must be dealing with some object or other.
   parse_pass_1_pov2_1(parse_object,WordList,Object,NWL),
   !,
   information(parse,object,Object),
   parse_pass_1_pov2(NWL,ReT).
% end top-level *****************

% First: whitespace eater
parse_pass_1_pov2_1(Token,[32|WordList], Result, NewWordList) :-
   !,
   parse_pass_1_pov2_1(Token, WordList, Result, NewWordList).

% include ***********************
% later... add 'restore earlier analysed include'
parse_include_pov2('colours.inc') :-
   !,
   consult('colours.pl').
parse_include_pov2('colors.inc') :-
   !,
   consult('colours.pl').
parse_include_pov2(FileName) :-
   flag(reading_file,Old,included),
   % assert(status(reading_file,included)), % assert new status
   parse_pov2(FileName),                  % parse include file
   % retract(status(reading_file,_)),       % retract new status
   % assert(status(reading_file,Status)).   % restore old status
   flag(reading_file,_,Old).
% include end *******************

% fog ***************************
% parse fog.   
parse_pass_1_pov2_1(parse_fog,WL,[colour(Col),distance(Dist)],NewWordList) :-
   parse_pass_1_pov2_1(left_brace_token,WL,_,WordList),
   list_string_member(WordList,[colour,color],_,NewWordList2),
   parse_pass_1_pov2_1(parse_colour,NewWordList2,Col,NWL2),
   list_string_match(NWL2,distance,NWL3),
   parse_pass_1_pov2_1(parse_float,NWL3,Dist,NWL4),
   parse_pass_1_pov2_1(right_brace_token,NWL4,_,NewWordList).
%parse_pass_1_pov2_1(parse_fog,WordList,colour(Result),NewWordList2) :-
%   list_string_member(WordList,[colour,color],_,NewWordList),
%   parse_pass_1_pov2_1(parse_colour,NewWordList,Result,NewWordList2).
%parse_pass_1_pov2_1(parse_fog,WordList,distance(Result),NewWordList2) :-
%   list_string_match(WordList,distance,NewWordList),
%   parse_pass_1_pov2_1(parse_distance,NewWordList,Result,NewWordList2).
% end fog ***********************

parse_pass_1_pov2_1(parse_texture,WL,Texture,NewWordList) :-
   parse_pass_1_pov2_1(left_brace_token,WL,_,WordList),
   information(token_list,'parse texture',WordList),nl,
   !,
   parse_pass_1_pov2_2(parse_texture,WordList,Texture,NewWordList),
   information(parse2,'texture',Texture).

parse_pass_1_pov2_2(parse_texture,WordList,[],NewWordList) :-
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NewWordList),
   !.
parse_pass_1_pov2_2(parse_texture,WordList,[Op|Ops],NewWordList) :-
   parse_pass_1_pov2_3(parse_texture,WordList,Op,NWL),
   !,
   information(parse2,'texture mod found',Op),
   parse_pass_1_pov2_2(parse_texture,NWL,Ops,NewWordList).

parse_pass_1_pov2_3(parse_texture,WordList,pigment(Op),NWL) :-
   list_string_match(WordList,pigment,NWL1),
   !,
   parse_pass_1_pov2_1(parse_texture_pigment,NWL1,Op,NWL).
parse_pass_1_pov2_3(parse_texture,WordList,finish(Op),NWL) :-
   list_string_match(WordList,finish,NWL1),
   !,
   parse_pass_1_pov2_1(parse_texture_finish,NWL1,Op,NWL).
parse_pass_1_pov2_3(parse_texture,WordList,normal(Op),NWL) :-
   list_string_match(WordList,normal,NWL1),
   !,
   parse_pass_1_pov2_1(parse_texture_normal,NWL1,Op,NWL).
parse_pass_1_pov2_3(parse_texture,WordList,translate(Op),NWL) :-
   list_string_match(WordList,translate,NWL1),
   !,
   parse_pass_1_pov2_1(parse_vector,NWL1,Op,NWL).
parse_pass_1_pov2_3(parse_texture,WordList,rotate(Op),NWL) :-
   list_string_match(WordList,rotate,NWL1),
   !,
   parse_pass_1_pov2_1(parse_vector,NWL1,Op,NWL).
parse_pass_1_pov2_3(parse_texture,WordList,scale(Op),NWL) :-
   list_string_match(WordList,scale,NWL1),
   !,
   parse_pass_1_pov2_1(parse_scale_vector,NWL1,Op,NWL).

% body generator ****************
parse_pass_1_pov2_body(_,WordList,[],NewWordList) :-
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NewWordList),
   !.
parse_pass_1_pov2_body(Wot,WordList,Mods,NewWordList) :-
   parse_pass_1_pov2_3(Wot,WordList,Op,NWL),
   parse_pass_1_pov2_body(Wot,NWL,Ops,NewWordList),
   Mods = [Op|Ops].
% end body generator ************

% pigment ***********************
parse_pass_1_pov2_1(parse_texture_pigment,WL,Texture,NewWordList) :-
   parse_pass_1_pov2_1(left_brace_token,WL,_,WordList),
   !,
   parse_pass_1_pov2_4(parse_texture_pigment,WordList,Texture,NewWordList).
   %parse_pass_1_pov2_body(parse_texture_pigment_body,WordList,Texture1,NWL),
   %!,
   %parse_pass_1_pov2_body(parse_texture_pigment_mod,NWL,Texture2,NewWordList),
   %append(Texture1,Texture2,Texture),
   %information(parse2,'found pigment: ',Texture).
   
% A pigment structure is _one_ pigment-'body', followed by any number
% of pigment modifications.   
parse_pass_1_pov2_4(parse_texture_pigment,WordList,Texture,NewWordList) :-
   parse_pass_1_pov2_3(parse_texture_pigment_body,WordList,Texture1,NWL),
   !,
   parse_pass_1_pov2_5(parse_texture_pigment_mod,NWL,Texture2,NewWordList),
   Texture = [Texture1|Texture2].
% Ok, so it must be a declared pigment
parse_pass_1_pov2_4(parse_texture_pigment,WordList,pigment(Result),NewWordList) :-
   bagof(Name,
      Inc^Def^declared(Inc,Name,pigment,Def),
      Declareds),
   list_string_member(WordList,Declareds,Result,NewWordList),
   !,
   information(parse,'Declared pigment',Result).

parse_pass_1_pov2_5(parse_texture_pigment_mod,WordList,[],NewWordList) :-
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NewWordList),
   !.
parse_pass_1_pov2_5(parse_texture_pigment_mod,WordList,[T|Ts],NewWordList) :-
   parse_pass_1_pov2_3(parse_texture_pigment_mod,WordList,T,NWL),
   parse_pass_1_pov2_5(parse_texture_pigment_mod,NWL,Ts,NewWordList).

parse_pass_1_pov2_3(parse_texture_pigment_body,WordList,colour(Col),NewWordList) :-
   list_string_member(WordList,[colour,color],_,NWL),
   !,
   % information(parse2,'colour',' '),
   parse_pass_1_pov2_1(parse_colour,NWL,Col,NewWordList).
parse_pass_1_pov2_3(parse_texture_pigment_body,WordList,Texture,NewWordList) :-
   list_string_member(WordList,[agate, bozo, granite, leopard, marble, onion,
                                painted1, painted2, painted3, spotted, wood,
                                radial ],Texture,NewWordList),
   information(parse2,'found pigment',Texture),
   !.
parse_pass_1_pov2_3(parse_texture_pigment_body,WordList,Texture,NewWordList) :-
   list_string_match(WordList,mandle,NWL),
   !,
   parse_pass_1_pov2_1(parse_float,NWL,Texture,NewWordList).
parse_pass_1_pov2_3(parse_texture_pigment_body,WordList,Texture,NewWordList) :-
   list_string_match(WordList,gradient,NWL),
   !,
   parse_pass_1_pov2_1(parse_vector,NWL,Texture,NewWordList).

/**************************************
parse_pass_1_pov2_2(parse_texture_pigment_mod,WordList,[],NewWordList) :-
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NewWordList).
parse_pass_1_pov2_2(parse_texture_pigment_mod,WordList,Mods,NewWordList) :-
   parse_pass_1_pov2_3(parse_texture_pigment_mod,WordList,Op,NWL),
   information(parse2,'texture_pigment mod found',Op),
   parse_pass_1_pov2_2(parse_texture_pigment_mod,NWL,Ops,NewWordList),
   Mods = [Op|Ops].
*************************************/

parse_pass_1_pov2_3(parse_texture_pigment_mod,WordList,scale(Scale),NewWordList) :-
   list_string_match(WordList,scale,NWL),
   !,
   information(parse2,'scale',' '),
   parse_pass_1_pov2_1(parse_scale_vector,NWL,Scale,NewWordList).
parse_pass_1_pov2_3(parse_texture_pigment_mod,WordList,quick_colour(Col),NewWordList) :-
   list_string_match(WordList,quick_colour,NWL),
   !,
   information(parse2,'quick_colour',' '),
   parse_pass_1_pov2_1(parse_colour,NWL,Col,NewWordList).
parse_pass_1_pov2_3(parse_texture_pigment_mod,WordList,colour_map(Col),NewWordList) :-
   list_string_member(WordList,[colour_map,color_map],_,NWL),
   !,
   information(parse2,'colour_map',' '),
   parse_pass_1_pov2_1(parse_colour_map,NWL,Col,NewWordList).
parse_pass_1_pov2_3(parse_texture_pigment_mod,WordList,Found,NewWordList) :-
   list_string_member(WordList,[translate,rotate],Op,NWL),
   !,
   parse_pass_1_pov2_1(parse_vector,NWL,Op2,NewWordList),
   Found =.. [Op,Op2].
parse_pass_1_pov2_3(parse_texture_pigment_mod,WordList,Found,NewWordList) :-
   list_string_member(WordList,[octaves,
                                omega,
                                lambda,
                                frequency,
                                phase,
                                agate_turb,
                                turbulence],Op,NWL),
   !,
   parse_pass_1_pov2_1(parse_float,NWL,Op2,NewWordList),
   information(parse3,Op,Op2),
   Found =.. [Op,Op2].
% end pigment *******************

% finish ************************
parse_pass_1_pov2_1(parse_texture_finish,WL,Texture,NewWordList) :-
   parse_pass_1_pov2_1(left_brace_token,WL,_,WordList),
   !,
   parse_pass_1_pov2_2(parse_texture_finish,WordList,Texture,NewWordList),
   true. % information(parse2,'found finish: ',Texture).
   
parse_pass_1_pov2_2(parse_texture_finish,WordList,[],NewWordList) :-
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NewWordList),
   !.
parse_pass_1_pov2_2(parse_texture_finish,WordList,[Op|Ops],NewWordList) :-
   parse_pass_1_pov2_3(parse_texture_finish,WordList,Op,NWL),
   !,
   information(parse2,'texture_finish mod found',Op),
   parse_pass_1_pov2_2(parse_texture_finish,NWL,Ops,NewWordList).
   % Mods = [Op|Ops].

parse_pass_1_pov2_3(parse_texture_finish,WordList,metallic,NewWordList) :-
   list_string_match(WordList,metallic,NewWordList),
   !.
parse_pass_1_pov2_3(parse_texture_finish,WordList,Found,NewWordList) :-
   list_string_member(WordList,[ambient,
                                brilliance,
                                diffuse,
                                reflection,
                                refraction,
                                ior,
                                phong,
                                phong_size,
                                specular,
                                roughness,
                                crand],Op,NWL),
   !,
   parse_pass_1_pov2_1(parse_float,NWL,Op2,NewWordList),
   Found =.. [Op,Op2].
% end finish ********************

% normal ************************
parse_pass_1_pov2_1(parse_texture_normal,WL,Texture,NewWordList) :-
   parse_pass_1_pov2_1(left_brace_token,WL,_,WordList),
   !,
   parse_pass_1_pov2_2(parse_texture_normal,WordList,Texture,NewWordList),
   true. % information(parse2,'found normal: ',Texture).
   
parse_pass_1_pov2_2(parse_texture_normal,WordList,[],NewWordList) :-
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NewWordList),
   !.
parse_pass_1_pov2_2(parse_texture_normal,WordList,[Op|Ops],NewWordList) :-
   parse_pass_1_pov2_3(parse_texture_normal,WordList,Op,NWL),
   !,
   information(parse2,'texture_normal mod found',Op),
   parse_pass_1_pov2_2(parse_texture_normal,NWL,Ops,NewWordList).
   % Mods = [Op|Ops].

parse_pass_1_pov2_3(parse_texture_normal,WordList,metallic,NewWordList) :-
   list_string_match(WordList,bump_map,NewWordList),
   !,
   information(parse2,'BUMPMAP NOT YET IMPLEMENTED!!!!',error),fail.
parse_pass_1_pov2_3(parse_texture_normal,WordList,Found,NewWordList) :-
   list_string_member(WordList,[bumps,
                                bumpy1,
                                bumpy2,
                                bumpy3,
                                dents,
                                ripples,
                                waves,
                                wrinkles],Op,NWL),
   !,
   parse_pass_1_pov2_1(parse_float,NWL,Op2,NewWordList),
   Found =.. [Op,Op2].

parse_pass_1_pov2_3(parse_texture_normal,WordList,Found,NewWordList) :-
   list_string_member(WordList,[turbulence,
                                translate,
                                rotate,
                                scale],Op,NWL),
   !,
   parse_pass_1_pov2_1(parse_vector,NWL,Op2,NewWordList),
   Found =.. [Op,Op2].
parse_pass_1_pov2_3(parse_texture_normal,WordList,Found,NewWordList) :-
   list_string_member(WordList,[octaves,
                                omega,
                                lambda,
                                frequency,
                                phase],Op,NWL),
   !,
   parse_pass_1_pov2_1(parse_float,NWL,Op2,NewWordList),
   Found =.. [Op,Op2].
% normal end ********************

% sphere ************************
parse_pass_1_pov2_1(parse_object,WL,
                    sphere([centre(Centre),radius(Radius)|Body]),NWL) :-
   list_string_match(WL,sphere,WL2),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL2,_,WordList),
   parse_pass_1_pov2_1(parse_vector,WordList,Centre,WL3),
   ( eat_comma(WL3,WL4) ; WL4 = WL3),
   parse_pass_1_pov2_1(parse_float,WL4,Radius,WL5),
   !,
   parse_pass_1_pov2_1(parse_object_mods,WL5,Body,WL6),
   information(parse1,'sphere',[centre(Centre),radius(Radius)|Body]),
   !,
   NWL = WL6.
   % parse_pass_1_pov2_1(right_brace_token,WL6,_,NWL),
   % write(got_here),nl.
% end sphere ********************

% box ***************************
parse_pass_1_pov2_1(parse_object,WL, box([bc(BC),tc(TC)|Body]),NWL) :-
   list_string_match(WL,box,WL2),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL2,_,WordList),
   parse_pass_1_pov2_1(parse_vector,WordList,BC,WL3),
   ( eat_comma(WL3,WL4) ; WL4 = WL3),
   parse_pass_1_pov2_1(parse_vector,WL4,TC,WL5),
   parse_pass_1_pov2_1(parse_object_mods,WL5,Body,NWL),
   information(parse1,'box',[bc(BC),tc(TC)|Body]),
   !.
% end box *************

% plane ***************
parse_pass_1_pov2_1(parse_object,WL,
                    plane([normal(Vect),dist(Float)|Body]),NWL) :-
   list_string_match(WL,plane,WL2),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL2,_,WordList),
   parse_pass_1_pov2_1(parse_vector,WordList,Vect,WL3),
   ( eat_comma(WL3,WL4) ; WL4 = WL3),
   parse_pass_1_pov2_1(parse_float,WL4,Float,WL5),
   !,
   parse_pass_1_pov2_1(parse_object_mods,WL5,Body,NWL),
   information(parse1,'plane',[normal(Vect),dist(Float)|Body]).
% end plane ***********

% cone ****************
parse_pass_1_pov2_1(parse_object,WL, cone([start(V1),F1,end(V2),F2|Body]),NWL) :-
   list_string_match(WL,cone,WL2),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL2,_,WordList),
   parse_pass_1_pov2_1(parse_vector,WordList,V1,WL3),
   ( eat_comma(WL3,WL4) ; WL3 = WL4),
   parse_pass_1_pov2_1(parse_float,WL4,F1,WL5),
   ( eat_comma(WL5,WL6) ; WL5 = WL6),
   parse_pass_1_pov2_1(parse_vector,WL6,V2,WL7),
   ( eat_comma(WL7,WL8) ; WL7 = WL8),
   parse_pass_1_pov2_1(parse_float,WL8,F2,WL9),
   parse_pass_1_pov2_1(parse_object_mods,WL9,Body,NWL),
   information(parse1,'cone',[V1,F1,V2,F2|Body]),
   !.
% end cone ************

% cylinder ************
parse_pass_1_pov2_1(parse_object,WL, cylinder([V1,V2,F|Body]),NWL) :-
   list_string_match(WL,cylinder,WL2),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL2,_,WordList),
   parse_pass_1_pov2_1(parse_vector,WordList,V1,WL3),
   ( eat_comma(WL3,WL4) ; WL3 = WL4),
   parse_pass_1_pov2_1(parse_vector,WL4,V2,WL5),
   ( eat_comma(WL5,WL6) ; WL5 = WL6),
   parse_pass_1_pov2_1(parse_float,WL6,F,WL7),
   (  list_string_match(WL7,open,WL8)
   -> parse_pass_1_pov2_1(parse_object_mods,WL8,Bod,NWL),
      Body = [open|Bod]
   ;  parse_pass_1_pov2_1(parse_object_mods,WL7,Body,NWL)
   ),
   information(parse1,'cylinder',[V1,V2,F|Body]),
   !.
% end cylinder ********

% disc ****************

% end disc ************

% quadric *************
parse_pass_1_pov2_1(parse_object,WL, quadric([V1,V2,V3,F|Body]),NWL) :-
   list_string_match(WL,quadric,WL1),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL1,_,WL2),
   parse_pass_1_pov2_1(parse_vector,WL2,V1,WL3),
   ( eat_comma(WL3,WL4) ; WL3 = WL4),
   parse_pass_1_pov2_1(parse_vector,WL4,V2,WL5),
   ( eat_comma(WL5,WL6) ; WL5 = WL6),
   parse_pass_1_pov2_1(parse_vector,WL6,V3,WL7),
   ( eat_comma(WL7,WL8) ; WL7 = WL8),
   parse_pass_1_pov2_1(parse_float,WL8,F,WL9),
   parse_pass_1_pov2_1(parse_object_mods,WL9,Body,NWL),
   information(parse1,'quadric',[V1,V2,V3,F|Body]),
   !.
% end quadric *********

% cubic ***************

% end cubic ***********

% poly ****************

% end poly ************

% torus ***************
parse_pass_1_pov2_1(parse_object,WL, torus([major(Maj),minor(Min)|Body]),NWL) :-
   list_string_match(WL,torus,WL2),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL2,_,WordList),
   parse_pass_1_pov2_1(parse_float,WordList,Maj,WL3),
   ( eat_comma(WL3,WL4) ; WL4 = WL3),
   parse_pass_1_pov2_1(parse_float,WL4,Min,WL5),
   parse_pass_1_pov2_1(parse_object_mods,WL5,Body,NWL),
   information(parse1,'torus',[major(Maj),minor(Min)|Body]),
   !.
% end torus ***********

% union ***************

% end union ***********

% merge ***************

% end merge ***********

% intersection ********

% end intersection ****

% difference **********

% end box *************

% bicubic_patch *******

% end bicubic_patch ***

% triangle ************
parse_pass_1_pov2_1(parse_object,WL, triangle([vertices(V1,V2,V3)|Body]),NWL) :-
   list_string_match(WL,triangle,WL2),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL2,_,WordList),
   parse_pass_1_pov2_1(parse_vector,WordList,V1,WL3),
   ( eat_comma(WL3,WL4) ; WL4 = WL3 ),
   parse_pass_1_pov2_1(parse_vector,WL4,V2,WL5),
   ( eat_comma(WL5,WL6) ; WL6 = WL5 ),
   parse_pass_1_pov2_1(parse_vector,WL6,V3,WL7),
   parse_pass_1_pov2_1(parse_object_mods,WL7,Body,NWL),
   information(parse1,'triangle',[vertices(V1,V2,V3)|Body]),
   !.
% end triangle ********

% smooth_triangle *****
parse_pass_1_pov2_1(parse_object,WL, 
           smooth_triangle([vertices(V1,V2,V3,V4,V5,V6)|Body]),NWL) :-
   list_string_match(WL,smooth_triangle,WL2),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL2,_,WordList),
   parse_pass_1_pov2_1(parse_vector,WordList,V1,WL3),
   ( eat_comma(WL3,WL4) ; WL4 = WL3 ),
   parse_pass_1_pov2_1(parse_vector,WL4,V2,WL5),
   ( eat_comma(WL5,WL6) ; WL6 = WL5 ),
   parse_pass_1_pov2_1(parse_vector,WL6,V3,WL7),
   ( eat_comma(WL7,WL8) ; WL8 = WL7 ),
   parse_pass_1_pov2_1(parse_vector,WL8,V4,WL9),
   ( eat_comma(WL9,WL10) ; WL10 = WL9 ),
   parse_pass_1_pov2_1(parse_vector,WL10,V5,WL11),
   ( eat_comma(WL11,WL12) ; WL11 = WL12 ),
   parse_pass_1_pov2_1(parse_vector,WL12,V6,WL13),
   parse_pass_1_pov2_1(parse_object_mods,WL13,Body,NWL),
   information(parse1,'smooth triangle',[vertices(V1,V2,V3,V4,V5,V6)|Body]),
   !.
% end smooth_triangle *

% height_field ********

% end height_field ****

% blob ****************

% end blob ************

% object **************

% end object **********

% declared object *****

% end declared object *

% Lightsource *********
parse_pass_1_pov2_1(parse_object,WL,
                    light_source([position(Centre),colour(Colour)|Body]),NWL) :-
   list_string_match(WL,light_source,WL2),
   !,
   parse_pass_1_pov2_1(left_brace_token,WL2,_,WordList),
   parse_pass_1_pov2_1(parse_vector,WordList,Centre,WL3),
   list_string_member(WL3,[colour,color],_,WL4),
   parse_pass_1_pov2_1(parse_colour,WL4,Colour,WL5),
   !,
   parse_pass_1_pov2_2(parse_lightsource,WL5,Body,NWL),
   information(parse1,'parse_lightsource',[position(Centre),colour(Colour)|Body]).

parse_pass_1_pov2_2(parse_lightsource,WL,[],NWL) :-
   parse_pass_1_pov2_1(right_brace_token,WL,_,NWL),
   !.
parse_pass_1_pov2_2(parse_lightsource,WL,[Op|Ops],NewWordList) :-
   parse_pass_1_pov2_3(parse_lightsource,WL,Op,NWL),
   %!,
   parse_pass_1_pov2_2(parse_lightsource,NWL,Ops,NewWordList).

parse_pass_1_pov2_3(parse_lightsource,WL,Op,NWL) :-
   list_string_member(WL,[point_at, translate, rotate],Match,WL2),
   !,
   parse_pass_1_pov2_1(parse_vector,WL2,Vector,NWL),
   Op =.. [Match|Vector].
parse_pass_1_pov2_3(parse_lightsource,WL,Op,NWL) :-
   list_string_member(WL,[tightness, radius, falloff, adaptive],Match,WL2),
   !,
   parse_pass_1_pov2_1(parse_float,WL2,Float,NWL),
   Op =.. [Match|Float].
parse_pass_1_pov2_3(parse_lightsource,WL,Match,NWL) :-
   list_string_member(WL,[track, jitter, spotlight],Match,NWL),
   !.
parse_pass_1_pov2_3(parse_lightsource,WL,scale(Vector),NWL) :-
   list_string_match(WL,scale,WL2),
   !,
   parse_pass_1_pov2_1(parse_scale_vector,WL2,Vector,NWL).
parse_pass_1_pov2_3(parse_lightsource,WL,looks_like(Object),NWL) :-
   list_string_match(WL,looks_like,WL2),
   !,
   parse_pass_1_pov2_1(parse_object,WL2,Object,NWL).
parse_pass_1_pov2_3(parse_lightsource,WL,area_light(V1,V2,F1,F2),NWL) :-
   list_string_match(WL,area_light,WL2),
   !,
   parse_pass_1_pov2_1(parse_vector,WL2,V1,WL3),
   eat_comma(WL3,WL4),
   parse_pass_1_pov2_1(parse_vector,WL4,V2,WL5),
   eat_comma(WL5,WL6),
   parse_pass_1_pov2_1(parse_float,WL6,F1,WL7),
   eat_comma(WL7,WL8),
   parse_pass_1_pov2_1(parse_float,WL8,F2,NWL).
% Lightsource end *****

% object modifications **
parse_pass_1_pov2_1(parse_object_mods,WordList,[],NewWordList) :-
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NewWordList),
   !.
parse_pass_1_pov2_1(parse_object_mods,WordList,Mods,NewWordList) :-
   information(debug,parse_object_mods_top,WordList),
   % write(WordList),nl,
   parse_pass_1_pov2_2(parse_object_mods,WordList,Op,NWL),
   information(debug,parse_object_mods_recurse,NWL),
   information(parse1,'object mod found',Op),
   !,
   parse_pass_1_pov2_1(parse_object_mods,NWL,Ops,NewWordList),
   Mods = [Op|Ops].

parse_pass_1_pov2_2(parse_object_mods,WordList,Modification,NWL) :-
   list_string_member(WordList,[translate,
                                rotate/*,
                                transform*/],Match,WL2),
   information(debug,parse_object_mods_transforms,WL2),
   !,
   parse_pass_1_pov2_1(parse_vector,WL2,Loc,NWL),
   Modification =.. [Match,Loc].
parse_pass_1_pov2_2(parse_object_mods,WordList,Match,NWL) :-
   list_string_member(WordList,[smooth, no_shadow, sturm, inverse],Match,NWL),
   !,
   information(debug,parse_object_mods_thingummies,NWL).
parse_pass_1_pov2_2(parse_object_mods,WordList,pigment(Bod),NWL) :-
   list_string_match(WordList,pigment,WL2),
   !,
   information(debug,parse_object_mods_pigment,WL2),
   parse_pass_1_pov2_1(parse_texture_pigment,WL2,Bod,NWL).
parse_pass_1_pov2_2(parse_object_mods,WordList,finish(Bod),NWL) :-
   list_string_match(WordList,finish,WL2),
   !,
   information(debug,parse_object_mods_finish,WL2),
   parse_pass_1_pov2_1(parse_finish,WL2,Bod,NWL).
parse_pass_1_pov2_2(parse_object_mods,WordList,normal(Bod),NWL) :-
   list_string_match(WordList,normal,WL2),
   !,
   information(debug,parse_object_mods_normal,WL2),
   parse_pass_1_pov2_1(parse_tnormal,WL2,Bod,NWL).
parse_pass_1_pov2_2(parse_object_mods,WordList,waterlevel(Bod),NWL) :-
   list_string_match(WordList,waterlevel,WL2),
   !,
   information(debug,parse_object_mods_waterlevel,WL2),
   parse_pass_1_pov2_1(parse_float,WL2,Bod,NWL).
parse_pass_1_pov2_2(parse_object_mods,WordList,texture(Bod),NWL) :-
   list_string_match(WordList,texture,WL2),
   !,
   information(debug,parse_object_mods_texture,WL2),
   parse_pass_1_pov2_1(parse_texture,WL2,Bod,NWL).
parse_pass_1_pov2_2(parse_object_mods,WordList,scale(Scale),NWL) :-
   list_string_match(WordList,scale,WL2),
   !,
   information(debug,parse_object_mods,WL2),
   parse_pass_1_pov2_1(parse_scale_vector,WL2,Scale,NWL).
parse_pass_1_pov2_2(parse_object_mods,WordList,[],NWL) :-
   information(debug,parse_object_mods,WordList),
   !,
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NWL).
% end object mods ********

% declared (this is gonna be fun) **
parse_pass_1_pov2_1(parse_declare,WordList,[Name,Wot,Def],NewWordList) :-
   % first; find the name. Does the current WordList start with a space?
   (  /* WordList=[32|WL] ;*/ WordList = WL ),
   % now split at the equals-sign
   append(N,[61|WL2],WL),
   !,
   (  append(N2,[32],N) ; N2 = N),
   string_to_list(Name,N2),
   parse_pass_1_pov2_1(parse_declare_2,WL2,[Wot,Def],NewWordList),
   true. % information(parse1,'declare',[Name,Wot,Def]).

parse_pass_1_pov2_1(parse_declare_2,WL,[colour,colour(Col)],NewWordList) :-
   list_string_member(WL,[colour,color],_,NewWordList2),
   !,
   parse_pass_1_pov2_1(parse_colour,NewWordList2,Col,NewWordList).
parse_pass_1_pov2_1(parse_declare_2,WL,[pigment,pigment(Col)],NewWordList) :-
   list_string_match(WL,pigment,NewWordList2),
   !,
   parse_pass_1_pov2_1(parse_texture_pigment,NewWordList2,Col,NewWordList).
parse_pass_1_pov2_1(parse_declare_2,WL,[normal,normal(Col)],NewWordList) :-
   list_string_match(WL,normal,NewWordList2),
   !,
   parse_pass_1_pov2_1(parse_texture_normal,NewWordList2,Col,NewWordList).
parse_pass_1_pov2_1(parse_declare_2,WL,[finish,finish(Col)],NewWordList) :-
   list_string_match(WL,finish,NewWordList2),
   !,
   parse_pass_1_pov2_1(parse_texture_finish,NewWordList2,Col,NewWordList).
parse_pass_1_pov2_1(parse_declare_2,WL,[camera,camera(Col)],NewWordList) :-
   list_string_match(WL,camera,NewWordList2),
   !,
   parse_pass_1_pov2_1(parse_camera,NewWordList2,Col,NewWordList).
parse_pass_1_pov2_1(parse_declare_2,WL,[colour_map,colour_map(Col)],NewWordList) :-
   list_string_match(WL,colour_map,NewWordList2),
   !,
   parse_pass_1_pov2_1(parse_colour_map,NewWordList2,Col,NewWordList).
parse_pass_1_pov2_1(parse_declare_2,WL,[texture,[texture(Te1)|Tes]],NewWordList) :-
   list_string_match(WL,texture,NewWordList2),
   !,
   % parse_pass_1_pov2_1(left_brace_token,WL2,_,WL3),
   information(token_list,NewWordList2),nl,
   parse_pass_1_pov2_1(parse_texture,NewWordList2,Te1,NewWordList3),
   parse_pass_1_pov2_1(parse_texture_declare,NewWordList3,Tes,NewWordList).
parse_pass_1_pov2_1(parse_declare_2,WL,[transform,transform(Col)],NewWordList) :-
   list_string_match(WL,transform,NewWordList2),
   !,
   parse_pass_1_pov2_1(parse_transform,NewWordList2,Col,NewWordList).
parse_pass_1_pov2_1(parse_declare_2,WL,[float,Float],NewWordList) :-
   %(  is_digit(H)  % it's something that starts with a digit - surely a float?
   %;  H=46),       % a float may also start with a fullstop, of course.
   %!,
   parse_pass_1_pov2_1(parse_float,WL,Float,NewWordList),
   !.
parse_pass_1_pov2_1(parse_declare_2,WL,[vector,Vector],NewWordList) :-
   parse_pass_1_pov2_1(parse_vector,WL,Vector,NewWordList),
   !.
parse_pass_1_pov2_1(parse_declare_2,WL,[object,Object],NewWordList) :-
   % information(parse1,'This had better be an object of some sort!',WL),
   parse_pass_1_pov2_1(parse_object,WL,Object,NewWordList),
   !.

parse_pass_1_pov2_1(parse_texture_declare,WordList,[texture(Te1)|Tes],NewWordList) :-
   list_string_match(WordList,texture,WL2),
   % parse_pass_1_pov2_1(left_brace_token,WL2,_,WL3),
   !,
   parse_pass_1_pov2_1(parse_texture,WL2,Te1,WL4),
   information(token_list,WL2),nl,
   information(token_list,WL4),nl,
   parse_pass_1_pov2_1(parse_texture_declare,WL4,Tes,NewWordList).
% so the above failed. return an empty list.
parse_pass_1_pov2_1(parse_texture_declare,WordList,[],WordList).
% declared end ***********

% end object ********************

% camera ************************
parse_pass_1_pov2_1(parse_camera,WL,Camera,NewWordList) :-
   parse_pass_1_pov2_1(left_brace_token,WL,_,WordList),
   !,
   parse_pass_1_pov2_2(parse_camera,WordList,Camera,NewWordList).

parse_pass_1_pov2_2(parse_camera,WordList,[],NewWordList) :-
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NewWordList),
   !.
parse_pass_1_pov2_2(parse_camera,WordList,Mods,NewWordList) :-
   parse_pass_1_pov2_3(parse_camera,WordList,Op,NWL),
   !,
   information(parse2,'camera mod found',Op),
   parse_pass_1_pov2_2(parse_camera,NWL,Ops,NewWordList),
   Mods = [Op|Ops].

parse_pass_1_pov2_3(parse_camera,WordList,Modification,NWL) :-
   list_string_member(WordList,[location,
                                direction,
                                up,
                                right,
                                sky,
                                look_at,
                                translate,
                                rotate/*,
                                transform*/],Match,WL2),
   !,
   parse_pass_1_pov2_1(parse_vector,WL2,Loc,NWL),
   Modification =.. [Match,Loc].
parse_pass_1_pov2_3(parse_camera,WordList,scale(Scale),NWL) :-
   list_string_match(WordList,scale,WL2),
   !,
   parse_pass_1_pov2_1(parse_scale_vector,WL2,Scale,NWL).
% end camera ********************

% colour ************************
%parse_pass_1_pov2_1(parse_colour,WordList,declared(Declared),NewWordList) :-
%   append(Declared,[A|NewWordList],WordList),
%   is_white(A),  ###################### uncomment once declared is sorted out
%   declared(_,Declared,colour,_).
parse_pass_1_pov2_1(parse_colour,WordList,rgb(R,G,B),NewWordList2) :-
   list_string_match(WordList,rgb,NewWordList),
   !,
   parse_pass_1_pov2_1(parse_vector,NewWordList,vector(R,G,B),NewWordList2).
parse_pass_1_pov2_1(parse_colour,WordList,rgbf(R,G,B,F),NewWordList2) :-
   list_string_match(WordList,rgbf,NewWordList),
   !,
   parse_pass_1_pov2_1(parse_vector,NewWordList,[R,G,B,F],NewWordList2).
parse_pass_1_pov2_1(parse_colour,WordList,Result,NewWordList) :-
   (  list_string_match(WordList,red,NWR)
   -> parse_pass_1_pov2_1(parse_float,NWR,R,NWR2)
   ;  R = 0,
      NWR2 = WordList
   ),
   information(parse2,red,R),
   (  list_string_match(NWR2,green,NWG)
   -> parse_pass_1_pov2_1(parse_float,NWG,G,NWG2)
   ;  G = 0,
      NWG2 = NWR2
   ),
   % information(parse2,green,G),
   (  list_string_match(NWG2,blue,NWB)
   -> parse_pass_1_pov2_1(parse_float,NWB,B,NWB2)
   ;  B = 0,
      NWB2 = NWG2
   ),
   % information(parse2,blue,B),
   (  list_string_match(NWB2,filter,NWF)
   -> parse_pass_1_pov2_1(parse_float,NWF,F,NewWordList)
   ;  F = 0,
      NewWordList = NWB2
   ),
   \+ (var(NWR), var(NWG), var(NWB), var(NWF)), % At least one needs to be specified!
   (  var(NWF)   % filter wasn't changed, hence only need rgb
   -> Result = rgb(R,G,B)
   ;  Result = rgbf(R,G,B,F)
   ).
parse_pass_1_pov2_1(parse_colour,WordList,Result,NewWordList) :-
   bagof(Name,
      Inc^Def^declared(Inc,Name,colour,Def),
      Declareds),
   list_string_member(WordList,Declareds,Result,NewWordList),
   information(parse,'Declared colour',Result).
% end colour ********************

% colour_map ********************
parse_pass_1_pov2_1(parse_colour_map,WordList,Col,NewWordList) :-
   parse_pass_1_pov2_1(left_brace_token,WordList,_,NWL),
   parse_pass_1_pov2_2(parse_colour_map,NWL,Col,NewWordList),
   !,
   information(parse,'Colour map',Col).
parse_pass_1_pov2_2(parse_colour_map,WordList,[],NewWordList) :-
   parse_pass_1_pov2_1(right_brace_token,WordList,_,NewWordList),
   !.
parse_pass_1_pov2_2(parse_colour_map,WordList,Cols,NewWordList) :-
   parse_pass_1_pov2_1(left_bracket_token,WordList,_,NWL1),
   !,
   parse_pass_1_pov2_1(parse_float,NWL1,F1,NWL2),
   ( eat_comma(NWL2,NWL22) ; NWL22 = NWL2),
   (  list_string_member(NWL2,[colour,color],_,NWL4)
   -> parse_pass_1_pov2_1(parse_colour,NWL4,C1,NWL5),
      list_string_member(NWL5,[colour,color],_,NWL6),
      parse_pass_1_pov2_1(parse_colour,NWL6,C2,NWL7),
      parse_pass_1_pov2_1(right_bracket_token,NWL7,_,NWL8),
      !,
      information(parse2,'Colour_map entry',[F1,F2,C1,C2]),
      parse_pass_1_pov2_2(parse_colour_map,NWL8,ColEntries,NewWordList),
      Cols = [colour_entry(F1,C1,C2)|ColEntries]
   ;  parse_pass_1_pov2_1(parse_float,NWL22,F2,NWL3),
      !,
      list_string_member(NWL3,[colour,color],_,NWL4),
      parse_pass_1_pov2_1(parse_colour,NWL4,C1,NWL5),
      list_string_member(NWL5,[colour,color],_,NWL6),
      parse_pass_1_pov2_1(parse_colour,NWL6,C2,NWL7),
      parse_pass_1_pov2_1(right_bracket_token,NWL7,_,NWL8),
      !,
      information(parse2,'Colour_map entry',[F1,F2,C1,C2]),
      parse_pass_1_pov2_2(parse_colour_map,NWL8,ColEntries,NewWordList),
      Cols = [colour_entry(F1,F2,C1,C2)|ColEntries]
   ).

% end colour_map  ********************

% parse rgbf
parse_pass_1_pov2_1(parse_rgbf,[60|FloatList],[Fx,Fy,Fz,FF],NewWordList) :-
   parse_pass_1_pov2_1(parse_float,FloatList,Fx,FLx),
   eat_comma(FLx,FLx2),
   parse_pass_1_pov2_1(parse_float,FLx2,Fy,FLy),
   eat_comma(FLy,FLy2),
   parse_pass_1_pov2_1(parse_float,FLy2,Fz,FLz),
   eat_comma(FLz,FLz2),
   parse_pass_1_pov2_1(parse_float,FLz2,FF,FLF),
   append(_,[62|NewWordList],FLF).

% parse vector
% -vector is UGLY    ##################################
parse_pass_1_pov2_1(parse_vector,[45|FloatList],ResVect,NewWordList) :-
   (  parse_pass_1_pov2_1(parse_vector,FloatList,vector(Fx,Fy,Fz),NewWordList)
   -> ResVect = vector(-Fx,-Fy,-Fz)
   ;  parse_pass_1_pov2_1(parse_vector,FloatList,vector(V),NewWordList),
      ResVect = vector(minus(V))
   ).    
parse_pass_1_pov2_1(parse_vector,[60|FloatList],vector(Fx,Fy,Fz),NewWordList) :-
   parse_pass_1_pov2_1(parse_float,FloatList,Fx,FLx),
   eat_comma(FLx,FLx2),
   parse_pass_1_pov2_1(parse_float,FLx2,Fy,FLy),
   eat_comma(FLy,FLy2),
   parse_pass_1_pov2_1(parse_float,FLy2,Fz,FLz),
   append(_,[62|NewWordList],FLz),
   !.
parse_pass_1_pov2_1(parse_vector,WordList,vector(Result),NewWordList) :-
   bagof(Name,
      Inc^Def^declared(Inc,Name,vector,Def),
      Declareds),
   list_string_member(WordList,Declareds,Result,NewWordList),
   !,
   information(parse1,'Declared vector',Result).
parse_pass_1_pov2_1(parse_vector,WordList,vector(Result),NewWordList) :-
   list_string_member(WordList,[x,y,z],Result,NewWordList),
   information(parse1,'Reserved vector',Result).

parse_pass_1_pov2_1(parse_scale_vector,FloatList,vector(Fx1,Fy1,Fz1),NewWordList) :-
   parse_pass_1_pov2_1(parse_vector,FloatList,vector(Fx,Fy,Fz),NewWordList),
   information(parse2,scale,[Fx,Fy,Fz]),
   !,
   (  Fx =:= 0
   -> message(errorWindow,'scaling by 0.0 Changed to 1.0 error! Changed to 1.0.'),
      Fx1 = 1
   ;  Fx1 = Fx
   ),
   (  Fy =:= 0
   -> message(errorWindow,'scaling by 0.0 Changed to 1.0 error! Changed to 1.0.'),
      Fy1 = 1
   ;  Fy1 = Fy
   ),
   (  Fz =:= 0
   -> message(errorWindow,'scaling by 0.0 Changed to 1.0 error! Changed to 1.0.'),
      Fz1 = 1
   ;  Fz1 = Fz
   ).

parse_pass_1_pov2_1(parse_float,FloatList,Result,NewFloatList) :-
   parse_pass_1_pov2_1(parse_float_term,FloatList,F,NFL1),
   !,
   (  NFL1 = [43|NFL2]                                           % +
   -> parse_pass_1_pov2_1(parse_float,NFL2,Fs,NewFloatList),
      Result = (F+Fs)
   ;  (  NFL1 = [45|NFL2]                                        % -
      -> parse_pass_1_pov2_1(parse_float,NFL2,Fs,NewFloatList),
         Result = (F-(Fs))
      ;  Result = F,
         NewFloatList = NFL1
      )
   ).
parse_pass_1_pov2_1(parse_float,FloatList,Result,NewFloatList) :-
   bagof(Name,
      Inc^Def^declared(Inc,Name,float,Def),
      Declareds),
   list_string_member(FloatList,Declareds,Result,NewFloatList),
   !,
   information(parse1,'Declared float',Result).
parse_pass_1_pov2_1(parse_float,FloatList,clock,NewFloatList) :-
   list_string_match(FloatList,clock,NewFloatList),
   !,
   information(parse1,'clock',' ' ).
parse_pass_1_pov2_1(parse_float,FloatList,version,NewFloatList) :-
   list_string_match(FloatList,version,NewFloatList),
   !,
   information(parse1,'version',' ' ).

%   parse_pass_1_pov2_1(parse_float,NFL2,Fs,NewFloatList),
%   Result =.. [Sign,F,Fs].
%parse_pass_1_pov2_1(parse_float,FloatList,F,NewFloatList) :-
%   parse_pass_1_pov2_1(parse_float_term,FloatList,F,NewFloatList).

parse_pass_1_pov2_1(parse_float_term,FloatList,Result,NewFloatList) :-
   parse_pass_1_pov2_1(parse_float_factor,FloatList,F,NFL1),
   information(parse1,float,F),
   (  NFL1 = [42|NFL2]                                           % *
   -> parse_pass_1_pov2_1(parse_float,NFL2,Fs,NewFloatList),
      Result = (F*Fs)
   ;  (  NFL1 = [47|NFL2]                                        % /
      -> parse_pass_1_pov2_1(parse_float,NFL2,Fs,NewFloatList),
         Result = (F/Fs)
      ;  Result = F,
         NewFloatList = NFL1
      )
   ).
%   (  parse_pass_1_pov2_1(multiply_token,NFL1,Op,NFL2)
%   ;  parse_pass_1_pov2_1(divide_token,NFL1,Op,NFL2)),
%   parse_pass_1_pov2_1(parse_float_term,NFL2,Fs,NewFloatList),
%   Result =.. [Op,F,Fs].
%parse_pass_1_pov2_1(parse_float_term,FloatList,Float,NewFloatList) :-
%   parse_pass_1_pov2_1(parse_float_factor,FloatList,Float,NewFloatList).

parse_pass_1_pov2_1(parse_float_factor,FloatList,Float,NewFloatList) :-
   parse_pass_1_pov2_1(parse_float_float,FloatList,Float,NewFloatList).

% Let's invent our own float grammar.
% float = fixed E Fixed | fixed e fixed | fixed
% fixed = integer[.integer] |
%         .integer
%parse_pass_1_pov2_1(parse_float_factor,FloatList,[F|Fs],NewFloatList) :-
%   true.
parse_pass_1_pov2_1(parse_float_float,FloatList,F,ReturnList) :-
   parse_pass_1_pov2_1(parse_float_fixed,FloatList,F1,FloatList2),
   (( FloatList2 = [101|FloatList3] ; FloatList2 = [69|FloatList3]  )
      % ( E = 101 , E = 69 )) % lower- and uppercase-e
   -> parse_pass_1_pov2_1(parse_float_fixed,FloatList3,F2,ReturnList),
      % so now we have F1 E F2.... convert this to prolog float.
      F is F1 * 10^F2
      %information(parse2,'float found',F)
   ;  % information(parse2,'float found',F1),
      F = F1,
      ReturnList = FloatList2
   ).
%parse_pass_1_pov2_1(parse_float_float,FloatList,F,ReturnList) :-
%   parse_pass_1_pov2_1(parse_float_fixed,FloatList,F,ReturnList).

parse_pass_1_pov2_1(parse_float_fixed,IntegerList,F,ReturnList) :-
   parse_pass_1_pov2_1(parse_integer,IntegerList,I1,IntegerList3),
   %information(parse2,'integer found',I1),
   (  IntegerList3 = [46|IntegerList2]
   -> parse_pass_1_pov2_1(parse_integer,IntegerList2,I2,ReturnList),
      shift_integer_behind_fullstop(I2,I3),
      F is I1 + I3
   ;  F is I1, %  * 1.00,
      ReturnList = IntegerList3
   ).
%parse_pass_1_pov2_1(parse_float_fixed,IntegerList,F,ReturnList) :-
%   parse_pass_1_pov2_1(parse_integer,IntegerList,F,ReturnList).

% ignore leading plus
/*parse_pass_1_pov2_1(parse_integer,[43,I|IntegerList],IR,ReturnList) :-
   is_digit(I),
   parse_pass_1_pov2_2(parse_integer,[I|IntegerList],IR,ReturnList).
   */
% leading minus
parse_pass_1_pov2_1(parse_integer,[45,I|IntegerList],IR,ReturnList) :-
   information(parse1,'minus integer!','-'),
   is_digit(I),
   !,
   parse_pass_1_pov2_2(parse_integer,[I|IntegerList],I2,ReturnList),
   IR is -I2.
parse_pass_1_pov2_1(parse_integer,[I|IntegerList],IR,ReturnList) :-
   is_digit(I),
   !,
   parse_pass_1_pov2_2(parse_integer,[I|IntegerList],IR,ReturnList).
%parse_pass_1_pov2_2(parse_integer,[I|ReturnList],-1,[I|ReturnList]) :-
%   !,
%   \+ is_digit(I).
parse_pass_1_pov2_2(parse_integer,[I|IntegerList],IR,ReturnList) :-
   (  is_digit(I)
   -> parse_pass_1_pov2_2(parse_integer,IntegerList,I2,ReturnList),
      (  I2 = -1
      -> IR is I-48
      ;  IR is (I-48)*10 + I2
      )
   ;  IR = -1,
      ReturnList = [I|IntegerList]
   ),!.   
% parse_pass_1_pov2_2(parse_integer,IntegerList,I1,[whatsadot|IntegerList2]),

% Make the integer into a list of 'digits', prefix with '0.', and convert back
shift_integer_behind_fullstop(I,IR) :-   
   term_to_atom(I,A1),
   string_to_atom(S1,A1),
   string_to_list(S1,IL),
   FL = [48,46|IL],        % prefix the list with '0.'
   string_to_list(S2,FL),
   string_to_atom(S2,A2),
   term_to_atom(IR,A2),
   !.                      % Surely that's enough?

eat_comma([44|Rest],Rest) :- !.
eat_comma([H|Tail],Rest) :- is_white(H),eat_comma(Tail,Rest).

/* the second pass.
**
**  the first pass will have included everything that needed inclusion,
**  and will have defined everything that needed defining.
*/
parse_pass_2_pov2([]) :- !, write('no more objects'),nl.
parse_pass_2_pov2([H|T]) :-
   flag(reading_file,Status,Status),
   (  Status = included
   -> message(warningWindow,'Non-declared object in included file. '),
      message(warningWindow,H)
   ;  true
   ),
   new_screen_object(H),
   !,
   parse_pass_2_pov2(T).

% list_string_member/4 succeeds if the top of the 'List' list
% is a member of arg 2. Returns the match and the rest of the 'List' list.
% sample usage:
%  list_string_member([65,66,67,68],['ABC',as,'AB'],{'ABC','AB'},{[68],[67,68]})
list_string_member([32|List], Members, Member, Tail) :-
   !,
   list_string_member(List, Members, Member, Tail).
/*list_string_member(List, [Member|Members], Result, Tail) :-
   atom(Member),
   !,
   list_atom_member(List,[Member|Members],Result,Tail).
   */
list_string_member(List, [Member|_], Member, Tail) :-
   %(  List2 = [32|List] ; List = List2 ),
   string_to_list(Member,ListMember),
   length(ListMember,Len),
   length(Dummy,Len),
   append(Dummy,Tail,List),
   Dummy=ListMember,
   information(parse2,member_match,Member),
   !.
list_string_member(List,[_|MemberList],Match,Tail) :-
   list_string_member(List,MemberList,Match,Tail).
/*
list_atom_member(List,[Atom|_],String,Tail) :-
   string_to_atom(String, Atom),
   list_string_match(List, String, Tail).
list_atom_member(List,[_|Ms],String,Tail) :-
   list_atom_member(List, Ms, String, Tail).
*/
list_string_match([32|List], Members, Tail) :-
   !,
   list_string_match(List, Members, Tail).
/*list_string_match(List, Match, Tail) :-
   atom(Match),
   !,
   list_atom_member(List,[Match],_,Tail).
*/
list_string_match(List2, Match, Tail) :-
   (  List2 = [32|List] ; List = List2),
   string_to_list(Match,ListMatch),
   length(ListMatch,Len),
   length(Dummy,Len),
   append(Dummy,Tail,List),
   Dummy=ListMatch,
   information(parse2,match,Match),
   !.
/*
list_string_match(List, Match, Tail) :-
   string_to_list(Match,ListMatch),
   length(ListMatch,Len),
   append(Head,Tail,List),
   length(Head,Len2),
   (  Len2 > Len
   -> fail
   ;  Len2 = Len
   ),
   Head = ListMatch.
   */
/********************************************************************/

/*
read_pov2_2(47,Chs) :-
   get0(42),   % ...so we have /, and should skip until the end of the comment
   !,
   read_comment_pov2(1),
   read_pov2(32,Chs).
read_pov2_3(B,Chs) :-
   ( B = 32 ; B = 9 ),   % space or tab
   get0(A),
   !,
   read_pov2_3(A,Chs).
read_pov2_3(C,[C|Cs]) :-
   get0(C2),
   read_pov2_2(C2,Cs).
*/


/*
read_token(Token) :-
   get(A),  % skip whitespaces
   get_rest_of_token_2(A,Token).
get_rest_of_token_3(Token) :-
   get0(A),
   get_rest_of_token_3(A,Token).
get_rest_of_token_3(A,[]) :-   
   \+ between(65,91,A),  % CAP
   \+ between(97,122,A), % lowercase
   \+ between(48,57,A),  % digit
   \+ A = 95,   % _
   \+ A = 10,   % newline?
   true. % to make it easy to add more as I remember them!

get_rest_of_token_2(32,error) :- !. % ' ' (space)
get_rest_of_token_2(123,[123]) :- !. % {
get_rest_of_token_2(125,[125]) :- !. % }
get_rest_of_token_2(60,[60]) :- !. % <
get_rest_of_token_2(62.[62]) :- !. % >
get_rest_of_token_2(34.Token) :- !. % "
   get_string_until_delimiter(34,Token)
   % get_rest_of_token_3(filename,Token),
   !.
get_rest_of_token_2(42.[42]) :- !. % *
get_rest_of_token_2(43.[43]) :- !. % +
get_rest_of_token_2(44.[44]) :- !. % ,
get_rest_of_token_2(45.[45]) :- !. % -
get_rest_of_token_2(47.[47]) :- !. % /
get_rest_of_token_2(A,[A|Token]) :-      % }
   between(65,91,A),
   get0(B),
   get_rest_of_token_3(B,Token),
   !.
get_rest_of_token_3(filename,Token) :-

get_string_until_delimiter(Delimiter, String) :-
   get0(A),
   get_string_until_delimiter_2(A,Delimiter,String).
get_string_until_delimiter_2(Delimiter,Delimiter,[]) :- !.
get_string_until_delimiter_2(A,Delimiter,[A|String]) :-
   get0(B),
   get_string_until_delimiter_2(B,Delimiter,String).
   */

parse_pass_1_pov2_1(plus_token,[43|NFL],plus,NFL) :- !.            % +
parse_pass_1_pov2_1(minus_token,[45|NFL],minus,NFL) :- !.          % -
parse_pass_1_pov2_1(multiply_token,[42|NFL],multiply,NFL) :- !.    % *
parse_pass_1_pov2_1(divide_token,[47|NFL],divide,NFL) :- !.        % /
parse_pass_1_pov2_1(left_par_token,[40|NFL],left_par,NFL) :- !.    % (
parse_pass_1_pov2_1(right_par_token,[41|NFL],right_par,NFL) :- !.  % )
parse_pass_1_pov2_1(hash_token,[35|NFL],hash,NFL) :- !.            % #
parse_pass_1_pov2_1(left_brace_token,[123,32|NFL],left_brace,NFL) :- !.
parse_pass_1_pov2_1(left_brace_token,[123|NFL],left_brace,NFL) :- !.% ,write('.....'),!.
parse_pass_1_pov2_1(left_brace_token,[32,123|NFL],left_brace,NFL) :- !.% ,write(',,,,,'),!.
parse_pass_1_pov2_1(right_brace_token,[32,125|NFL],right_brace,NFL) :- write(yoho),nl,!.
parse_pass_1_pov2_1(right_brace_token,[125|NFL],right_brace,NFL) :- !.% write('.....'),!.
parse_pass_1_pov2_1(left_bracket_token,[91|NWL],left_bracket_token,NWL) :- !.
%parse_pass_1_pov2_1(left_bracket_token,[32,91|NWL],left_bracket_token,NWL) :- !.
%parse_pass_1_pov2_1(right_bracket_token,[93,32|NWL],right_bracket_token,NWL) :- !.
parse_pass_1_pov2_1(right_bracket_token,[93|NWL],right_bracket_token,NWL) :- !.
%parse_pass_1_pov2_1(_token,[47|NFL],divide,NFL) :- !.

/*
% Ah. We couldn't parse whatever was given us. Tell the user and quit.
parse_pass_1_pov2(Parsing,TokenList,_,_) :-
   write('Ah. Parse error. Parsing at top level: '), write(Parsing), nl,
   (( length(TokenList,Len), Len < 68 )
   -> List = TokenList
   ;  length(List,68),
      append(List,_,TokenList)
   ),
   write('TokenList: '), string_to_list(String,List), write(String),
   !,
   fail.
parse_pass_1_pov2_1(Parsing,TokenList,_,_) :-
   write('Ah. Parse error. Parsing at level 1: '), write(Parsing), nl,
   (( length(TokenList,Len), Len < 68 )
   -> List = TokenList
   ;  length(List,68),
      append(List,_,TokenList)
   ),
   write('TokenList: '), string_to_list(String,List), write(String),
   !,
   fail.
parse_pass_1_pov2_2(Parsing,TokenList,_,_) :-
   write('Ah. Parse error. Parsing at level 2: '), write(Parsing), nl,
   (( length(TokenList,Len), Len < 68 )
   -> List = TokenList
   ;  length(List,68),
      append(List,_,TokenList)
   ),
   write('TokenList: '), string_to_list(String,List), write(String),
   !,
   fail.
parse_pass_1_pov2_3(Parsing,TokenList,_,_) :-
   write('Ah. Parse error. Parsing at level 3: '), write(Parsing), nl,
   (( length(TokenList,Len), Len < 70 )
   -> List = TokenList
   ;  length(List,70),
      append(List,_,TokenList)
   ),
   write('TokenList: '), string_to_list(String,List), write(String),
   !,
   fail.
  */ 
