spline(Type, ControlPoints, NoPoints, NewPoints) :-
   member(Type,[bezier,b_spline,cr_spline]),
   length(ControlPoints,NoCP),
   NoPoints > NoCP,      %% surely there are more linesegments wanted than 
   NoCP > 3,             %% there are controlpoints???
   DeltaT is (NoCP-3.0)/(NoPoints-1), %% trust me, I know what I'm doing - S. Hammer
   (  NoPoints < 30*NoCP              %% can we use the precalculated coeffs?
   -> spline_2(Type, ControlPoints, 0, DeltaT, NewPoints)       %% yes.
   ;  writef('Expensive spline calculation... Hang on tight..\n'),
      spline_3(Type, ControlPoints, 0, DeltaT, NewPoints)       %% no.
   ).
spline_2(_Type,CPs,_T,_DeltaT,[]) :-
   length(CPs,L),
   L < 4,
   !.
spline_2(Type,[CP1,CP2,CP3,CP4|CPs],T,DeltaT,[P1|NPs]) :-
   T2 is integer(T*100),
   calc_spline(Type,T2,CP1,CP2,CP3,CP4,P1),
   TN is T+DeltaT,
   (  TN < 1.0001
   -> spline_2(Type,[CP1,CP2,CP3,CP4|CPs],TN,DeltaT,NPs)
   ;  TN2 is TN - 1.0,
      spline_2(Type,[CP2,CP3,CP4|CPs],TN2,DeltaT,NPs)
   ).
spline_3(_Type,CPs,_T,_DeltaT,[]) :-
   length(CPs,L),
   L < 4,
   !.
spline_3(hermite,[CP1,CP2,CP3,CP4|CPs],T,DeltaT,[P|NPs]) :-
   (( CP1=[CX1,CY1,CZ1], CP2=[CX2,CY2,CZ2], CP3=[CX3,CY3,CZ3], CP4=[CX4,CY4,CZ4]) 
   ;( CP1=vector(CX1,CY1,CZ1), CP2=vector(CX2,CY2,CZ2), 
      CP3=vector(CX3,CY3,CZ3), CP4=vector(CX4,CY4,CZ4))),
   T2 is T*T,
   T3 is T2*T,
   P1 is 2*T3-3*T2+1,
   P2 is -2*T3+2*T2,
   P3 is T3-2*T2+T,
   P4 is T3-T2,
   PX is CX1*P1+CX2*P2+CX3*P3+CX4*P4,
   PY is CY1*P1+CY2*P2+CY3*P3+CY4*P4,
   PZ is CZ1*P1+CZ2*P2+CZ3*P3+CZ4*P4,
   P = [PX,PY,PZ],
   TN is T+DeltaT,
   (  TN < 1.0001
   -> spline_3(hermite,[CP1,CP2,CP3,CP4|CPs],TN,DeltaT,NPs)
   ;  TN2 is TN - 1.0,
      spline_3(hermite,[CP2,CP3,CP4|CPs],TN2,DeltaT,NPs)
   ).
spline_3(bezier,[CP1,CP2,CP3,CP4|CPs],T,DeltaT,[P|NPs]) :-
   (( CP1=[CX1,CY1,CZ1], CP2=[CX2,CY2,CZ2], CP3=[CX3,CY3,CZ3], CP4=[CX4,CY4,CZ4]) 
   ;( CP1=vector(CX1,CY1,CZ1), CP2=vector(CX2,CY2,CZ2), 
      CP3=vector(CX3,CY3,CZ3), CP4=vector(CX4,CY4,CZ4))),
   T2 is 1.0*T*T,
   T_1 is 1.0-T,
   T_12 is T_1*T_1,
   T_13 is T_12*T_1,
   P1 is T_13,
   P2 is 3.0*T*T_12,
   P3 is 3.0*T2*T_1,
   P4 is T2*T,
   PX is CX1*P1+CX2*P2+CX3*P3+CX4*P4,
   PY is CY1*P1+CY2*P2+CY3*P3+CY4*P4,
   PZ is CZ1*P1+CZ2*P2+CZ3*P3+CZ4*P4,
   P = [PX,PY,PZ],
   TN is T+DeltaT,
   (  TN < 1.0001
   -> spline_3(bezier,[CP1,CP2,CP3,CP4|CPs],TN,DeltaT,NPs)
   ;  TN2 is TN - 1.0,
      spline_3(bezier,[CP2,CP3,CP4|CPs],TN2,DeltaT,NPs)
   ).
spline_3(b_spline,[CP1,CP2,CP3,CP4|CPs],T,DeltaT,[P|NPs]) :-
   (( CP1=[CX1,CY1,CZ1], CP2=[CX2,CY2,CZ2], CP3=[CX3,CY3,CZ3], CP4=[CX4,CY4,CZ4]) 
   ;( CP1=vector(CX1,CY1,CZ1), CP2=vector(CX2,CY2,CZ2), 
      CP3=vector(CX3,CY3,CZ3), CP4=vector(CX4,CY4,CZ4))),
   T2 is T*T,
   T3 is T2*T,
   P1 is (-T3+3.0*T2-3.0*T+1.0)/6.0,
   P2 is (3.0*T3-6.0*T2+4.0)/6.0,
   P3 is (-3.0*T3+3.0*T2+3.0*T+1.0)/6.0,
   P4 is T3/6.0,
   PX is CX1*P1+CX2*P2+CX3*P3+CX4*P4,
   PY is CY1*P1+CY2*P2+CY3*P3+CY4*P4,
   PZ is CZ1*P1+CZ2*P2+CZ3*P3+CZ4*P4,
   P = [PX,PY,PZ],
   TN is T+DeltaT,
   (  TN < 1.0001
   -> spline_3(b_spline,[CP1,CP2,CP3,CP4|CPs],TN,DeltaT,NPs)
   ;  TN2 is TN - 1.0,
      spline_3(b_spline,[CP2,CP3,CP4|CPs],TN2,DeltaT,NPs)
   ).
spline_3(cr_spline,[CP1,CP2,CP3,CP4|CPs],T,DeltaT,[P|NPs]) :-
   (( CP1=[CX1,CY1,CZ1], CP2=[CX2,CY2,CZ2], CP3=[CX3,CY3,CZ3], CP4=[CX4,CY4,CZ4]) 
   ;( CP1=vector(CX1,CY1,CZ1), CP2=vector(CX2,CY2,CZ2), 
      CP3=vector(CX3,CY3,CZ3), CP4=vector(CX4,CY4,CZ4))),
   T2 is T*T,
   T3 is T2*T,
   P1 is (-T3+2*T2-T)/2.0,
   P2 is (3*T3-5*T2+2)/2.0,
   P3 is (-3*T3+4*T2+T)/2.0,
   P4 is (T3-T2)/2.0,
   PX is CX1*P1+CX2*P2+CX3*P3+CX4*P4,
   PY is CY1*P1+CY2*P2+CY3*P3+CY4*P4,
   PZ is CZ1*P1+CZ2*P2+CZ3*P3+CZ4*P4,
   P = [PX,PY,PZ],
   TN is T+DeltaT,
   (  TN < 1.0001
   -> spline_3(cr_spline,[CP1,CP2,CP3,CP4|CPs],TN,DeltaT,NPs)
   ;  TN2 is TN - 1.0,
      spline_3(cr_spline,[CP2,CP3,CP4|CPs],TN2,DeltaT,NPs)
   ).
   
calc_spline(Type,T,vector([CX1,CY1,CZ1]), vector([CX2,CY2,CZ2]),
            vector([CX3,CY3,CZ3]), vector([CX4,CY4,CZ4]),P) :-
   !,
   spline_coefficient(Type,T,P1,P2,P3,P4),
   PX is CX1*P1+CX2*P2+CX3*P3+CX4*P4,
   PY is CY1*P1+CY2*P2+CY3*P3+CY4*P4,
   PZ is CZ1*P1+CZ2*P2+CZ3*P3+CZ4*P4,
   P = [PX,PY,PZ].
calc_spline(Type,T,[CX1,CY1,CZ1],[CX2,CY2,CZ2],[CX3,CY3,CZ3],[CX4,CY4,CZ4],P) :-
   spline_coefficient(Type,T,P1,P2,P3,P4),
   PX is CX1*P1+CX2*P2+CX3*P3+CX4*P4,
   PY is CY1*P1+CY2*P2+CY3*P3+CY4*P4,
   PZ is CZ1*P1+CZ2*P2+CZ3*P3+CZ4*P4,
   P = [PX,PY,PZ].

